
module GT

! F90 Modul for the calculation of the PERCEIVED TEMPERATURE
!
! provided by DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human Biometeorology
!
! Concept and code written by Dr. Henning Staiger
! put to F90 module by Dr. Stefan Muthers
! 
! Reference: 
!   Staiger, H., Laschewski, G. & Grtz, A., 2012. The perceived temperature - a
!   versatile index for the assessment of the human thermal environment. Part A:
!   scientific basics. International Journal of Biometeorology, 56: 165-176.
!
! Date of code version: 2020-02-20
!

implicit none
save

contains
!
!---------------------------------------------------------------------
      real function tgef_vdi(linkepath,breite,laenge,hoehe,                       &
     &                      jahr,monat,tag,utc,                         &
     &                      tluft,taupunkt,ff,vau1m,nges,ntief,cl,cm,ch,&
     &                      anemoheight,tmrt,el2m)
!---------------------------------------------------------------------
!
! Funktionswert: gefuehlte Temperatur, Grad Celsius: Typ real
!                Definition der gefuehlten Temperatur siehe Function pt_basic.
!
! Version fuer eine synoptische Beobachtung oder eine numerische Vorhersage
! des Wetterzustandes, wobei dann die Wolkengattungen der 3 Stockwerke
! den vorhergesagten Bedeckungsgrad (Achtel) statt der Wolkenart enthalten.
!
! Eingangsgroessen der Argumentliste:
! -----------------------------------
! - linkepath : Pfadname zu den Linke Monatswerten aus dem europ. Atlas 
!               zur Sonnenstrahlung
! Geographie:
! - breite : geogr. Breite in dezimalen Graden, + = N, - = S: Typ real
! - laenge : geogr. Laenge in dezimalen Graden, + = E, - = W: Typ real
! - hoehe  : Hoehe ueber NN in m: Typ real
! Datum und Uhrzeit:
! - jahr   : Jahreszahl inklusiv Jahrhundert, z.B. 1998: Tapinteger
! - monat  : Monat: Typ integer
! - tag    : Tag  : Typ integer
! - utc    : Zeit in dezimalen Stunden (z.B 13.25 = 13 Uhr 15 Minuten), UTC:
!            Typ real
! Variablen aus der synoptischen Beobachtung/numerischen Prognose:
! - tluft      : Lufttemperatur 2 m ueber Grund, Grad Celsius: Typ real
! - taupunkt   : Taupunkttemperatur 2 m ueber Grund, Grad Celsius: Typ real
! - ff         : Windgeschwindigkeit in Hoehe des Anemometers: Typ real
! - iw         : Einheit von ff, siehe "interne steuernde Variablen":
!                Typ integer
! - anemoheight: Hoehe des Anemometers ueber Grund, m: Typ real
! - nges       : Gesamtbedeckunggrad beobachtet/vorhergesagt, Achtel,
!                erlaubte Werte 0 bis 8, ggf. 9 fuer Nebel, FM12: Typ integer
! - ntief      : Bedeckunggrad mit tiefen/mittelhohen Wolken beobachtet/
!                vorhergesagt, Achtel, FM 12, Werte 0 bis 8 <= ngrd:
!                Typ integer
! - cl         : Tiefe Wolken: Typ integer
!                - beobachtet: Typ der tiefen Wolken nach FM12
!                - vorhergesagt: Bedeckungsgrad mit tiefen Wolken, Achtel
! - cm         : mittelhohe Wolken: Typ integer
!                - beobachtet: Typ der mittelhohen Wolken nach FM12
!                - vorhergesagt: Bedeckungsgrad mit mittelhohen Wolken, Achtel
! - ch         : Hohe Wolken: Typ integer
!                - beobachtet: Typ der hohen Wolken nach FM12
!                - vorhergesagt: Bedeckungsgrad mit hohen Wolken, Achtel
! Steuerung:
! - nprog      : Angabe ob Beobachtung oder Vorhersage: Typ integer
!                - 0 entspricht Beobachtung nach FM12
!                - 1 entspricht numerische Vorhersage
! - nerr       : Fehlerstatus: Typ integer
!                - falls < 0 Fehler (keine Konvergenz etc., s. pt_basic)
!                - falls >=0 Ergebnis o.k., Wert entspricht Zahl der
!                  Iterationen
!
! Interne steuernde Variable:
! ---------------------------
! - iw         : Einheit der Windgeschwindigkeit von "ff" nach VuB2, Bd. D,
!                Tab. 1855: iw= 0, 1= m/s, iw= 3, 4 = Knoten, iw > 5= km/h
! - anemoheight: Hoehe des Anemometers ueber Grund, Meter
!
! Output Variablen:
! -----------------
! - ptc        : Gefuehlte Temperatur klassisch (keine Korrekturen kalt / warm)
! - schwuel    : String: Schwuelebedingung
!
! Benoetigte Funktionen:
! ----------------------
! - juliandat  : Tagesnummer (1 bis 365/366): Typ integer
! - fgammdek   : Winkel der Sonnenhoehe, Grad: type real
! - dampfdruck : Wasserdampfsaettigungsdruck, hPa: Typ real
! - wind_1m    : Umrechnung der Windgeschwindigkeit ff in Anemometerhoehe
!                auf Hoehe in 1 m ueber Grund und Einheit m/s: Typ real
! - ftmrt_g    : Mittlere Strahlungstemperatur, Grad Celsius: Typ real
! - pt_basic   : Berechnung der gefuehlten Temperatur mit Basis-Eingangs-
!                groessen. Type real. Jetzt mit 'Ridder'
!
! Erforderliche Unterprogramme:
! -----------------------------
! - synoprad   : Berechnung der lang- und kurzwelligen Strahlungsfluesse nach
!                VDI 3789/2 aus einer synoptische Beobachtung
!
! English Version:
! ================
!
! value of function tgef_vdi is the perceived temperature, degree
!                 centigrade: type real
!                 Definition of perceived temperature see function pt_basic
!
! Version for a synoptic observation according to WMO-FM12 or numerically
! predicted weather condition. For prediction the genus of clouds in
! observation must be replaced by the amount of clouds (octa) in the ceilings
! low, medium, and high.
!
! Input variables:
! ----------------
! - linkepath  : path to the linke map file (global monthly mean values)
! Geographic location:
! - breite     : geographic latitude,  decimal degree, north = +, south = -:
!                type real.
! - laenge     : geographic longitude, decimal degree, east  = +, west  = -:
!                type real.
! - hoehe      : altitude above mean sea level, meter: type real
! Date and time:
! - jahr       : year : century included, e.g. 1998: type integer
! - monat      : month: type integer
! - tag        : day  : type integer
! - utc        : time : decimal hours, unit: UTC (Universal Time Coordinated):
!                type real
! Variables of synoptic observation or predicted weather conditions:
! - tluft      : air temperature 2 m above ground level, degree centigrade:
!                type real.
! - taupunkt   : dew point temperature 2 m above ground level, degree
!                centigrade: type real
! - ff         : Wind speed in height of the anemometer, unit is knots (iw=4):
!                type real:
! - iw           unit of ff: possible values see "internal steering variables":
!                type integer
! - anemoheight  the height of the anemometer above ground level, m: type real
! - nges       : degree of total cloudiness, octa: 0 to 8, 9 = fog:
!                type integer.
! - ntief      : degree of cloudiness in the ceilings low or medium
!                octa: 0 to 8 <= nges: type integer
! - cl         : low ceiling: type integer
!                - observed:   genus of low    clouds (0 to 9)
!                - forecasted: cloud cover in the ceiling low, octa
! - cm         : medium ceiling: type integer
!                - observed:   genus of medium clouds (0 to 9)
!                - forecasted: cloud cover in the ceiling medium, octa
! - cl         : high ceiling: type integer
!                - observed:   genus of high   clouds (0 to 9)
!                - forecasted: cloud cover in the ceiling high, octa
! Steering:
! - nprog      : indication synoptic observation or numeric forecast
!                type integer. Cases:
!                - 0= synoptic observation
!                - 1= numeric forecast
! - nerr       : error status: type integer
!                - <  0, error: iteration doesn't converge etc., see pt_basic
!                - >= 0, o.k., value = count of steps for convergence
!
! Internal steering variables:
! ----------------------------
! - iw         : Unit of wind speed according to WMO-FM12; type integer
!                (GER: VuB2, Bd. D, Tab. 1855)
!                Unit of wind speed.:iw=0,1= m/s, iw=3,4=knots, iw >5=km/h
!                Defaults: iw=1= m/s measured, iw=4= knots measured
! - anemoheight: Anemometer height above ground level, meter; type real
!
! Output variables:
! -----------------
! - ptc        : classical perceived temperature (no adjustment cold / warm)
! - schwuel    : string: sultriness
!
! Functions required:
! -------------------
! - juliandat  : day number of the year: type integer
! - fgammdek   : solar altitude angle, degree: type real
! - dampfdruck : water vapour saturation pressure: type real
! - wind_1m    : transformation of measured wind speed in wind speed 1 m above
!                ground level: type real
! - ftmrt_g    : mean radiatiant temperature, degree centigrade : type real
! - pt_basic   : calculation of perceived temperature, degree C: type real
!
! Subroutine required:
! --------------------
! - synoprad   : calculation of the components of short-wave and long-wave
!                radiation from a synoptic observation (VDI 3789/2)
!
!**********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!**********************************************************************
!
!
      implicit none
!
!     --- Typ der bereitzustellenden synoptischen Beobachtungswerte
!         type of required synoptic observation
      integer jahr,monat,tag,nges,ntief,cl,cm,ch,nerr
      real tluft,taupunkt,ff
      real breite,laenge,hoehe,utc
!
!     --- Typ der Variablen des Programms/type of required programme variables
      integer iw, nprog
      parameter (iw=0)
      parameter (nprog=0)
      real anemoheight
      character meas*1
      character linkepath*(*)
      integer julia,ngrd,nh,ncl,ncm,nch
      real dek,tt2m,td,el2m,vau1m,tmrt,pb,pmva,clo,pmvs
      real gamma,difiso,difzrk,dirson_b,g_kw,r_kw,a_lw,e_lw,gh_kw,dh_kw
      character schwuel*1
      real ptc,d_std
!
!     --- Typ der benoetigten Funktionen/type of used functions
!      integer juliandat
!      real fgamdek,dampfdruck,wind_1m,ftmrt_g,pt_basic,fpboden
!
!     --- Initialisierung
      if(nprog.eq.0) then
         meas(1:1)='m'
      else
         meas(1:1)='p'
      endif
!
!     --- Bewoelkungsgroessen/cloudiness
!         Umsetzung - Wertebereich pruefen/transformation - test on definition
      ncl=cl
      if(ncl.gt.8) ncl=8
      if(ncl.lt.0) ncl=0
      ncm=cm
      if(ncm.gt.8) ncm=8
      if(ncm.lt.0) ncm=0
      nch=ch
      if(nch.gt.8) nch=8
      if(nch.lt.0) nch=0
      ngrd=nges
      if(ngrd.gt.9) ngrd=9
      if(ngrd.lt.0) ngrd=0
!
      if(meas(1:1).ne.'m'.and.meas(1:1).ne.'M') then
!        --- Numerische Prognose: Bedeckung tiefe/mittelhohe Wolken nach MAP
!            (Bedeckung mit Index l,m)
!            Numeric forecast: Cloudiness of the ceilings low and medium by the
!            procedure of MAP
         nh=int((float(ncl)/8.+(1.-float(ncl)/8.)*float(ncm)/8.)*8.+    &
     &           0.0001)
      else
!        --- Synoptische Beobachtung - Wertebereich pruefen
!            synoptic observation - test on definition
         nh=ntief
         if(nh.gt.ngrd) nh=ngrd
         if(nh.lt.0) nh=0
         if(ngrd.eq.9) nh=9
      endif
!
!     --- Julianischer Tag und Sonnenhoehe in Abhaengigkeit von
!         Datum, Koordinaten und Uhrzeit/gamma=angle of sun altitude, degree
      julia=juliandat(jahr,monat,tag)
      gamma=fgamdek(breite,laenge,julia,utc,dek)
!
!     --- Lufttemperatur 2m/air temperature 2m above ground level
      tt2m=tluft
!
!     --- Taupunkttemperatur 2m/dewpoint temperature 2m above ground level
      td=taupunkt
      if(td.gt.tt2m) td=tt2m
!
!     --- abgeleitete Feuchtemasse: el2m= Wasserdampfdruck (hPa)
!         calculation of water vapour pressure (hPa)
      el2m = Dampfdruck(td)
!
!     --- Wind in 1 m Hoehe/transformation of wind speed to 1 m above ground
!         meas in allen Faellen auf "p" setzen, beruecksichtigt rel. Wind 
!         durch metabolische Rate
      vau1m=wind_1m(ff,iw,anemoheight,meas)
!
!     --- Strahlung (VDI3789/2) aus Synop/calculate radiation components
      call synoprad(linkepath,julia,breite,laenge,gamma,hoehe,          &
     &   tt2m,vau1m,ngrd,nh,ncl,ncm,nch,meas,                           &
     &   difiso,difzrk,dirson_b,g_kw,r_kw,a_lw,e_lw)
      gh_kw=dirson_b+difzrk+difiso
      dh_kw=difiso
!
!     --- mittlere Strahlungstemperatur/mean radiation temperature
      tmrt=ftmrt_g(gamma,gh_kw,dh_kw,r_kw,a_lw,e_lw)
!
!     --- Luftdruck aus Hoehe ueber NN, hPa
!         calculate barometric pressure, hPa, at altitude above msl
      pb=fpboden(hoehe)
!
!     --- Funktionswert der gefuehlte Temperatur/value of function tgef_vdi
      tgef_vdi=pt_basic(tt2m,el2m,tmrt,vau1m,pb,pmva,                   &
     &                  clo,nerr,ptc,schwuel,d_std,pmvs)
!
      return
      END function tgef_vdi
!
!---------------------------------------------------------------------
      integer function juliandat (jahr,monat,tag)
!---------------------------------------------------------------------
!
! JULIAN.FOR
!
! The value of the function JULIANDAT is the day number of the year
! (Jan. 1st=1 to Dec. 31st= 365/366): type integer.
!
!     Der julianische Tag des Jahres wird aus dem Datum berechnet.
!
! Argumentlist:
! -------------
! - jahr  : year, century included, i.e. 1998: type integer
! - monat : month: type integer
! - tag   : day: type integer
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human Biometeorology 
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typ der Argumente / type of arguments
      integer jahr,monat,tag
!
!     --- Typ der programminternen Variablen 
!         type of programme variables
      integer mon(12)
!
      data mon /0,31,59,90,120,151,181,212,243,273,304,334/
!
!     --- Funktionswert / value of the function
      juliandat=mon(monat)+tag
      if(mod(jahr,400).eq.0.or.                                         &
     &  (mod(jahr,4).eq.0.and.mod(jahr,100).ne.0)) then
            if(monat.gt.2) then
               juliandat=juliandat+1
            endif
      endif
!
      return
      END function juliandat
!
!---------------------------------------------------------------------
      real function fgamdek(breite,laenge,julia,utc,dek)
!---------------------------------------------------------------------
!
! FGAMDEK.FOR         identisch mit fgamma, Deklination in der Argumentliste
!
! The function value is the solar altitude angle, degree, calculated from
! geographic coordinates (breite= latitude, laenge= longitude), day number
! of the year (julian ), and time (utc). In the variable "dek" the sun
! declination is returned. A sun altitude beneath the horizon is set to nil.
!
!     Berechnet den Hoehenwinkel h der Sonne aus geogr. Koordinaten, Tag und
!     Uhrzeit. Hoehenwinkel unter dem astr. Horizont = 0 gesetzt
!
! Argument list:
! --------------
! Input:
! ------
! - breite = geographic latituden, decimale degrees (north = +, south = -):
!            type real.
! - laenge = geographic longitude, decimale degrees (east  = +, west = -):
!            type real.
! - julia  = day number of the year (Jan. 1st=1 to Dec. 31st= 365/366):
!            type integer.
! - utc    = time in decimale hours in Universal Time Coordinated (UTC):
!            type real.
! Output:
! -------
! - fgamdek= solar altitude angle, degree: value of the function: type real.
! - dek    = sun declination, degree: type real.
!
! Included functions:
! -------------------
! - Wahreortszeit (true local time): type real.
! - Declination: type real.
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! source Klima-Michel by  Prof. Dr. Gerd Jendritzky
! code by Dr. Henning Staiger
!*********************************************************************
!
      implicit none
!
!     --- Umrechnungfaktor fuer Grad auf Radians (360 = 2*pi)
!         conversion factor degree to radians (360 degree = 2*pi)
      real pi,rad
      parameter (pi=3.141593, rad=pi/180.)
!
!     --- Typvereinbarung fuer Argumentliste / type of arguments
      real breite, laenge, utc, singam
      integer julia
!
!     --- Typvereinbarungen fuer das Programm
!         type of programme variables
      real dek, woz, vie, dlt, tau
!
!     --- Typen der Funktionen / type of required functions
!      real Wahreortszeit
!      real Declination
!
      dek = Declination (julia)
      woz = Wahreortszeit (utc, laenge, julia)
      vie = breite * rad
      dlt = dek * rad
      tau = 15. * (woz - 12.) * rad
!
!     --- Berechnung der Sonnenhoehe / calculates sun elevation
      singam = SIN (vie) * SIN (dlt) + COS (vie) * COS (dlt) * COS (tau)
!
!     --- Funktionswert / value of the function
      fgamdek = ASIN (singam) / rad
      return
      END function fgamdek
!
!---------------------------------------------------------------------
      real FUNCTION Declination (julia)
!---------------------------------------------------------------------
!
!     Die Deklination (Neigung der Aequatorebene zur Ekliptik) wird als
!     Funktion des julianischen Datums berechnet.
!
!     calculate declination as function of day number of the year (julia)
!
      implicit none
!
      real  x, sind
      INTEGER julia
!
!     --- Umrechnungfaktor fuer Grad auf Radians (360 = 2*pi)
!         conversion factor degree to radians (360 degree = 2*pi)
      real pi,rad
      parameter (pi=3.141593, rad=pi/180.)
!
      x = .9856 * julia - 2.72
      sind = .3978 * SIN ((x - 77.51 + 1.92 * SIN (x * rad)) * rad)
!
!     --- Funktionswert / value of the function
      declination = ASIN (sind) / rad
!
      return
      END function Declination
!
!---------------------------------------------------------------------
      real FUNCTION Wahreortszeit (utc, laenge, julia)
!---------------------------------------------------------------------
!
! Berechnung der wahren Ortszeit (true local time) aus UTC, geogr. Laenge,
! julianisches Datum julia
!
! calculate true local time: Input time in UTC, geogr. longitude, degree,
! day number of the year (julia)
!
      implicit none
!
      real laenge, utc, x, z
      INTEGER julia
!
!     --- Umrechnungfaktor fuer Grad auf Radians (360 = 2*pi)
!         conversion factor degree to radians (360 degree = 2*pi)
      real pi,rad
      parameter (pi=3.141593, rad=pi/180.)
!
      x = (.9856 * julia - 2.72)
!
!     --- Zeitgleichung z in Minutenm / equation of time z, minutes
!         
      z = - 7.66 * SIN (x * rad) - 9.87 * SIN ((2 * x + 24.99 + 3.83 *  &
     &    SIN (x * rad)) * rad)
!
!     --- Funktionswert / value of the function
      wahreortszeit = utc + (laenge * 4. + z) / 60.
!
      return
      END function Wahreortszeit
!
!-----------------------------------------------------------------------
      real function Dampfdruck(tt)
!-----------------------------------------------------------------------
!
! DAMPFDRUCK.F90
!
! The value of the function is the saturation water vapour pressure 
! (hPa = hecto Pascal) for a given temperature tt (C).
! For example, tt can be the air temperature or the dew point temperature.
!
! Method:
! -------
! Based on WMO recommendations: Constants a, b, c derived from 
! "Aspirations-Psychrometertafel", Deutscher Wetterdient DWD (1976). 
! Saturation water vapour pressure over ice is omitted.
!
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! source Klima-Michel by Prof. Dr. Gerd Jendritzky
! code by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
      real      :: tt, a, b, c
!
      a = 6.1078
!
      if(tt.lt.0.) then
!        ... tt  < 0 (C): saturation water vapour pressure over water
         b = 17.84362
         c = 245.425
      else
!        ... tt >= 0 (C): saturation water vapour pressure over water
         b = 17.08085
         c = 234.175
      endif
!
!     --- value of the function
      dampfdruck = a * exp(b * tt / (c + tt))
!
      return
      end function Dampfdruck

!
!---------------------------------------------------------------------
      real function wind_1m(ff,iw,anemoheight,meas)
!---------------------------------------------------------------------
!
! FWIND_1M.FOR
!
! Transforms wind speed in height of anemometer in wind speed 1 m above
! ground level. Unit of function value is m/s. Wind speed 1 m above ground
! level is required as input for subroutine FANGER/function PT_BASIC.
!
! Input arguments:
! ----------------
! - ff         : Winds peed in height of the anemometer, unit see iw;
!                type real
! - iw         : Unit of wind speed according to WMO-FM12; type integer
!                (GER: VuB2, Bd. D, Tab. 1855)
!                Unit of wind speed.:iw=0,1=m/s, iw=3,4= knots, iw>5= km/h
!                Defaults: iw=1= m/s measured, iw=4= knots measured
! - anemoheight: Anemometer height above ground level, meter; type real
! - meas       : indication whether ff is measured or predicted:
!                Type character*1. Cases:
!                - 'm' or 'M'= ff is measured: function wind_1m can return
!                  values lower than 0.5 m/s
!                - 'p' or all others without 'm' or 'M': ff is predicted:
!                  lowest value function wind_1m returns is 0.5 m/s
!
! Function value:
! ---------------
! wind_1m      : wind speed in 1 m above ground level, m/s: Type real
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology 
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Umrechnung Knoten und km/h auf m/s, Wert von z0
!         coefficients for transformation to the unit m/s,
!         z0 = value of roughness length, m
      real knots,kmh,z0
      parameter (knots=1.852/3.6, kmh=1./3.6, z0=0.01)
!
!     --- Typ der Eingangsgroessen der Argumentliste
      character meas*(*)
      real ff,anemoheight
      integer iw
!
!     --- Typ der Variablen des Programms
      real ffhoch,vau1m
!
!     --- Wind von Anemometerhoehe auf 1 m Hoehe reduzieren
!         transform wind speed in anemoheight to a wind speed 1 m above ground
      ffhoch=anemoheight
      if(ffhoch.lt.0.1) ffhoch=10.
      vau1m=ff*alog(1./z0)/alog(ffhoch/z0)
!
!     --- auf Dimension m/s umrechnen/transform to unit of wind speed
      if(iw.gt.2) then
         if(iw.le.5) then
            vau1m=vau1m*knots
         else
            vau1m=vau1m*kmh
         endif
      endif
!
!     --- Bei Prognosen Mindestgeschw. in 1 m Hoehe auf 0.4 m/s setzen
!         in case of numeric forecast: set to a minimal wind speed
      if(meas(1:1).ne.'m'.and.meas(1:1).ne.'M') then
         if(vau1m.lt.0.4) vau1m=0.4
      endif
!
!     --- Funktionswert/value of function wind_1m
      wind_1m=vau1m
!
      return
      END function wind_1m
!
!---------------------------------------------------------------------
      subroutine synoprad(linkepath,julia,breite,laenge,gamma,hoehe,    &
     &   tt2m,vau1m,ngrd,nh,ncl,ncm,nch,meas,                           &
     &   difiso,difzrk,dirson_b,g_kw,r_kw,a_lw,e_lw)
!---------------------------------------------------------------------
!
! SYNOPRAD.FOR
!
! Calculates components of short-, and long-wave radiation on the basis of a
! synoptic observation. Radiation calculation according to VDI 3789 / Part 2.
!
! Steuert Berechnung der aus synoptischen Eingangsgroessen abgeleiteten
! Strahlungskomponenten nach VDI 3789/2.
!
! Argumentlist:
! -------------
! Input variables:
! ----------------
! - linkepath  : path to the linke map file
! - julia      : day number of the year (1 to 365/366): type integer.
! - gamma      : solar altitude angle, degree: type real.
! - hoehe      : location altitude above mean sea level, m: type real.
! - tt2m       : air temperature 2 m above ground level, degree centigrade:
!                type real.
! - vau1m      : Wind speed 1 m above ground level, m/s: type real
!                type real
! - ngrd       : degree of total cloudiness, octa: 0 to 8, 9 = fog:
!                type integer.
! - nh         : degree of cloudiness with low, middle, or high clouds,
!                octa: 0 to 8: type integer
! - ncl        : ceiling low: type integer
!                - Observation: genus of low    clouds (0 to 9)
!                - Prediction : amount of cloud cover, octa
! - ncm        : ceiling medium: type integer
!                - Observation: genus of medium clouds (0 to 9)
!                - Prediction : amount of cloud cover, octa
! - nch        : ceiling high: type integer
!                - Observation: genus of high   clouds (0 to 9)
!                - Prediction : amount of cloud cover, octa
! - meas       : case observed/predicted: type character*1
!                - 'm' = observed
!                - 'p' = predicted
!
! Output variables:
! -----------------
! - difiso  : isotropic component of diffuse solar radiation onto a
!             horizontal surface (short-wave), W m-2: type real.
! - difzrk  : anisotropic component of diffuse solar radiation onto a
!             horizontal surface (short-wave), W m-2: type real.
! - dirson_b: direct solar radiation onto a horizontal/inclined surface
!             (short-wave), W m-2: type real.
! - g_kw    : global solar radiation onto a horizontal surface (short-wave),
!             W m-2: type real.
!             For inclined surfaces use dirson_b and add the diffusive
!             components.
! - r_kw    : reflected global radiation (short-wave), W m-2: type real
! - a_lw    : thermal radiation of the atmosphere (long-wave),
!             W m-2: type real
! - e_lw    : thermal radiation of the earth surface inclusive reflected part
!             (long-wave), W m-2: type real
!
! Function included:
! ------------------
! - fpboden.for : air pressure in location altitude, hPa = hecto Pascal:
!                 type real.
!
! Subroutines included:
! ---------------------
! - wolklvdi: defines degree of cloudines of low, middle and high clouds,
!             respectively, from a synoptic observation; input for lwstvdi
! - kwstvdi : short-wave radiation components according to VDI 3789 / Part 2,
!             functions included:
!             - ftlvdi  : Linke turbidity factor: type real.
!             - fdr0m   : relative optical air mass and vertical optical
!                         thickness: type real.
!             - fi0abstk: relative instantenous distance of the earth
!                         from the sun
! - lwstvdi : long-wave radiation components according to VDI 3789 / Part 2,
!             function included:
!             - tboden : temperature of the earth surface, Kelvin: type real.
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Umrechnungfaktor fuer Grad auf Radian (360 Grad = 2*pi),
!         transform degree in radian
      real pi,rad
      parameter (pi=3.141593, rad=pi/180.)
!
!     --- Typvereinbarungen fuer die Eingangsgroessen der Argumentliste
!         types of argumentlist input
      integer julia,ngrd,nh,ncl,ncm,nch
      real gamma,hoehe,tt2m,vau1m
      character meas*(*)
      character linkepath*(*)
!
!     --- Typvereinbarungen fuer die Ausgabegroessen der Argumentliste
!         types of argumentlist output
      real difiso,difzrk,dirson_b,g_kw,r_kw,a_lw,e_lw
!
!     --- Typvereinbarungen fuer weitere Strahlungsgroessen und Wolkenschichten
!         type of further radiation and ceiling variables
!         --- KWSTVDI
      real albedo,dirsnk
!         --- LWSTVDI und WOLKLVDI
      real em,tbc,xnl,xnm,xnh
      integer lndntzg
!
!     --- Typvereinbarungen fuer Variable des Programms
!         types of programme variables
      real pb,singam,qk,breite,laenge
      integer neff
!
!     --- Typvereinbarungen fuer die benoetigten Funktionen
!         types of requested functions
!      real fpboden
!
!     --- Initialisierung/initialization:
!         albedo = kurzwelliger Rueckstreuanteil des Bodens/albedo of surface
!         lndntzg= Landnutzung (1= Freiflaeche, 2= Siedlung,   3= Wasser)
!                  soil types  (1= pasture,     2= settlement, 3= sea)
!         em     = Emissionszahl des Erdbodens/emission coefficient of surface
      data albedo /0.2/, lndntzg /1/, em /0.95/
!
!     --- Luftdruck aus Hoehe ueber NN, hPa
!         calculate local air pressure, hPa, from height above msl
      pb = fpboden(hoehe)
!
!     --- Sinus der Sonnenhoehe/sinus of sun altitude
      singam=sin(rad*gamma)
!
!     --- Einflussgroessen der Bewoelkung auf die Strahlung bereitstellen
!         calculate cloudiness in 3 ceilings from synoptic observation
      call wolklvdi(ngrd,nh,ncl,ncm,nch,meas,                           &
     &              xnl,xnm,xnh)
!
!     --- Kurz- und langwellige Strahlungsfluesse/short and longwave radiation
!     ========================================================================
      if(meas(1:1).ne.'m'.and.meas(1:1).ne.'M') then
!        --- Num. Prognose: Effektiver statt Gesamtbedeckungsgrad
!            Num. forecast: effective total cloudiness
         neff=int((float(ngrd)+nh)*0.5+0.0001)
      else
!        --- Gesamtbedeckung aus Synop/total cloudiness from synop. observation
         neff=ngrd
      endif
!     --- kurzwellige Strahlungsfluesse/short wave radiation components
      call kwstvdi(linkepath,julia, breite,laenge, gamma, albedo, pb, neff, nh,   &
     &             difiso, difzrk, dirsnk ,dirson_b, g_kw, r_kw)
!
!     --- kurzwellige Strahlungsbilanz/short wave net radiation
      qk=g_kw-r_kw
!
!     --- langwellige Strahlungsfluesse/long wave radiation components
      call lwstvdi(tt2m,vau1m,qk,em,lndntzg,xnl,xnm,xnh,                &
     &             tbc,a_lw,e_lw)
!
      return
      END subroutine synoprad
!
!---------------------------------------------------------------------
      real function fpboden(hoehe_nn)
!---------------------------------------------------------------------
!
! FPBODEN.F
!
! fpboden : barometric pressure at site altitude estimated from the
!           site altitude (hoehe_nn, m) itself (ICAO atmosphere)
!
! Funktionswert:
! fpboden: Luftdruck an der Station, hPa, aus Stationshoehe ueber NN
!
! Argument: hoehe_nn= Stationshoehe, m, ueber NN
!
! VDI-3789/Blatt 2 (1994) (F. Kasten): Modell zur Berechnung der
! kurzwelligen Strahlungsfluesse ohne Horizonteinschraenkung
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
      real hoehe_nn,p0
      parameter (p0=1013.25)
!
!     --- Funktionswert / value of the function
      fpboden=p0*exp(-hoehe_nn/8434.5)
!
      return
      END function fpboden
!
!---------------------------------------------------------------------
      subroutine wolklvdi(ngrd,nh,cl,cm,ch,meas,                        &
     &                   xnl,xnm,xnh)
!---------------------------------------------------------------------
!
! WOLKLVDI.FOR
!
! Festlegung des Bedeckungsgrades in den drei Wolkenstockwerken anhand
! der Synop-Wolkengruppe 8NhClCmCh.
!
! Fixes the amount of cloud cover in the ceilings low, medium, high
!
! VDI 3789/2 fordert zur Parametrisierung der langwelligen Waermestrahlung
! der Atmosphaere den Bedeckungsgrad in den 3 Schichten als Real-Zahl:
!
! Literatur / references: 
!            VDI (1994): Umweltmeteorologie, Wechselwirkungen zwischen
!            Atmosphaere und Oberflaechen. Berechnung der kurz- und lang-
!            welligen Strahlung. VDI-Richtlinie 3789, Blatt 2
!
!            Kasten F (1989): Strahlungsaustausch zwischen Oberflaechen
!            und Atmosphaere. VDI-Berichte
!
! Eingangsgroessen der Argumentliste/input from argumentlist:
! - ngrd    : Gesamtbedeckungsgrad (0 bis 8/8, 9= Nebel), 
!             total cloud cover (0 to 8 octa, 9= fog)
! - nh/ntief: Bedeckungsgrad der tiefen/mittelhohen Wolkenschicht (0 bis 8/8)
!             partial cloud cover due to low and/or medium clouds
! - cl      : Art der tiefen Bewoelkung      bei Prognose: Bedeckung Okta
!             type of low clouds,    in case of predictions: cloud cover, octa
! - cm      : Art der mittelhohen Bewoelkung bei Prognose: Bedeckung Okta
!             type of medium clouds, in case of predictions: cloud cover, octa
! - cm      : Art der hohen Bewoelkung       bei Prognose: Bedeckung Okta
!             type of high clouds,   in case of predictions: cloud cover, octa
! - meas    : Unterscheidung Beobachtung , 'm', oder Vorhersage, 'p'
!             differentiate between observation , 'm', and prediction, 'p'
!
! Ausgabegroessen der Argumentliste/output to argumentlist:
! - xnl     : Bedeckung mit tiefen Wolken     : 0.0 bis 1.0 
!             cloud cover by low clouds       : 0.0 to  1.0
! - xnm     : Bedeckung mit mittelhohen Wolken: 0.0 bis 1.0
!             cloud cover by medium clouds    : 0.0 to  1.0
! - xnh     : Bedeckung mit hohen Wolken      : 0.0 bis 1.0
!             cloud cover by high clouds      : 0.0 to  1.0
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typvereinbarungen fuer die Argumentliste / type of arguments
      integer ngrd,nh,cl,cm,ch
      character meas*(*)
      real xnl,xnm,xnh
!
!     --- Typvereinbarungen der Variablen des Programms
!         type of programme variables
      integer ndelta,ntief
!
!     --- Initialisieren / initialise
      xnl=0.
      xnm=0.
      xnh=0.
!
!     --- Fall numerische Vorhersage: Bedeckung, octa, liegt bereits vor
!         case numeric predicted cloudiness in 3 ceilings (octa)
      if(meas(1:1).ne.'m'.and. meas(1:1).ne.'M') then
         xnl=float(cl)/8.
         xnm=float(cm)/8.
         xnh=float(ch)/8.
         return
      endif
!
!     --- Bedeckungsgrade der 3 Wolkenstockwerke definieren, um die
!         Ueberlappungen der Schichten beruecksichtigen zu koennen
!         (Kasten 1989)
!         case synoptic observation: account for overlapping cloud 
!         cover in the 3 ceilings
      ntief=nh
      if(ntief.gt.ngrd) ntief=ngrd
      ndelta=ngrd-ntief
!
!     --- ngrd=0 oder Beobachtungs-/Meldefehler / total cloud cover = 0
      if(ngrd.le.0.or.ngrd.gt.9) goto 200
!
!     --- Nebel wie 8/8 tiefe Wolken behandeln / handle fog as low clouds
      if(ngrd.eq.9.or.ntief.eq.9) then
         xnl=1.
         goto 200
      endif
!
!     --- Fall keine tiefen/mittelhohen Wolken: ntief=0
!         case: no low/medium clouds: ntief=0
      if(ntief.le.0.or.ntief.gt.8) then
         xnh=float(ngrd)/8.
         goto 200
      endif
!
!     ------ Faelle ntief > 0
!     ... Faelle (cases) ntief > 0 und/and cl = 0
      if(cl.eq.0) then
         if(ch.eq.0) then
            xnm=float(ngrd)/8.
         else
            xnh=float(ngrd)/8.
            xnm=float(ntief)/8.
         endif
         goto 200
      endif
!     --- Faelle (cases) ntief > 0 und/and cl > 0
      if(ch.eq.0) then
         if(cm.eq.0) then
            xnl=float(ngrd)/8.
         else
            xnm=float(ngrd)/8.
            xnl=float(ntief)/8.
         endif
      else
         if(cm.eq.0) then
            xnh=float(ngrd)/8.
            xnl=float(ntief)/8.
         else
            xnh=(float(ntief)+float(ndelta)/2.)/8.
            xnm=xnh
            xnl=float(ntief)/8.
         endif
      endif
!
  200 continue
!
      return
      END subroutine wolklvdi
!
!---------------------------------------------------------------------
      SUBROUTINE kwstvdi(linkepath,julia, breite, laenge,gamma, albedo, &
     &                   pb, ngrd, nh,                                  &
     &                   difiso, difzrk, dirsnk ,dirson_b, g_kw, r_kw)
!---------------------------------------------------------------------
!
! KWSTVDI.FOR:
!
! VDI-3789/Blatt 2 (1994) (F. Kasten): Modell zur Berechnung der
! kurzwelligen Strahlungsfluesse ohne Horizonteinschraenkung
!
! Literatur / refernces: 
!            VDI (1994): Umweltmeteorologie, Wechselwirkungen zwischen
!            Atmosphaere und Oberflaechen. Berechnung der kurz- und lang-
!            welligen Strahlung. VDI-Richtlinie 3789, Blatt 2
!
! The programme calculates direct, isotropic diffuse and reflected short-wave
! solar radiation without limitation of the horizon. 
!
! Required input:
! ---------------
! - linkepath : path to the linke map file
! - julia : day number of the year
! - gamma : solar elevation, degree, 
! - albedo: albedo of the ground for short-wave irradiation: 0 - 1
! - pb    : barometric pressure at site altitude, hPa
! - ngrd  : total cloud cover, octa (9: fog is handled as 8 octa low clouds)
! - nh    : partial cloud cover from low/medium cloud, octa
! - trueb : factor to multiply the standard deviation (stdv) of a mean Linke 
!           turbidity factor (TL0): TL = TL0 +/-trueb*stdv - 16.4.2013 no
!           longer required
! Output:
! -------
! - difiso: isotropic diffuse irradiation on a horizontal surface:         W/m2
! - difzrk: ansisotropic part of diff irradiation on a horizontal surface: W/m2
! - dirsnk: direct solar irradiation on surface normal to solar elevation: W/m2
! - dircon_b: direct solar irradiation on a horizontal surface:            W/m2
! - g_kw  : short-wave global solar irradiation on a horizontal surface:   W/m2
! - r_kw  : isotropic reflected solar irradiation on a horizontal surface: W/m2
!
! Required functions:
! -------------------
! - fi0abstk: extraterrestrial solar irradiation corr. for earth sun distance
! - fdr0m   : relative optical air mass and vertical optical thickness
! - tl_julia  : Linke turbidity factor - daily mean - derived from the 
!               monthly mean values in the European Solar Radiation Atlas
!
! Das Programm berechnet die direkte, diffuse und reflektierte Sonnenstrahlung
! unter dem Einfluss von
! - Sonnenhoehe gamma,
! - Hoehe ueber NN (pb),
! - Truebung und nach
! - Gesamtbedeckungsgrad ngrd (0 bis 8, 9 wird wie 8 behandelt)
!
! Modifikationen gegenueber der Version VDI 3789/2:
! - Linke'scher Truebungskoeffizienten werden dem Europ. Solar Radiation Atlas 
!   entnommen
! - Beruecksichtigung hoher Bewoelkung mit Gesamtbedeckungsgrad NGRD groesser
!   als Bedeckungsgrad mit tiefen/mittelhohen Wolken Nh
!
! Ausgabegroessen:
! - dirsnk = direkte Sonnenstrahlung auf zur Einstrahlung senkrechte Flaeche,
!            W m-2 in Abhaengigkeit vom Bedeckungsgrad
! - dirson_b= direkte Sonnenstrahlung auf die horizontale/geneigte Empfangs-
!            flaeche in Abhaengigkeit vom Bedeckungsgrad, W m-2
! - g_kw  =  Globalstrahlung nur auf die horizontale Flaeche,
!            W m-2: Fuer geneigte Flaechen dirson_b verwenden und diffus
!            addieren
! - difiso = (isotrope) diffuse (Himmel-) Strahlung auf die horizontale/
!            geneigte Flaeche, W m-2
! - difzrk = zirkumsolarer Teil der diffusen Himmelstrahlung, W m-2,
!            auf die horizontale/geneigte Flaeche
! - r_kw   = reflek. Strahlung auf die nach unten (!) orientierte horizon-
!            tale Flaeche (kurzwellig, nur isotrop), W m-2.
!hier
! - r_kw_o = reflekt. Strahlung auf die geneigte/horizontale nach oben (!)
!            orientierte Flaeche, W m-2, inaktiviert
! - gespiegelte sowie anisotrope und isotrope reflektierte Anteile auf die
!   geneigte/horizontale Flaeche (Formeln 26 a-c) werden nicht behandelt.
!
!hier
! Die horizontale Empfangsflaeche ist ein Sonderfall der geneigten Flaeche.
! Da ueblicherweise fuer die horizontale Flaeche gerechnet wird, werden die
! benoetigten Winkel (cosaega=cos(aeta)/sin(gamma) und cos2b_2=cos2(beta/2))
! fuer diesen Fall in einem Parameterstatement fest vorgegeben. Um fuer
! geneigte Flaechen rechnen zu koennen, sind die notwendigen Winkelrechnungen
! im Programm enthalten, aber auskommentiert. Sie sind mit Suchen von "chier"
! aufzufinden. Um sie aktivieren zu koennen sind als Eingangsgroessen der
! Argumentliste zusaetzlich erforderlich:
! - alpha = Azimuthwinkel, Grad, der Normale auf die geneigte Flaeche, Sued= 0,
!           nach W positiv, nach E negativ
! - beta  = Neigungswinkel, Grad, der geneigten Flaeche gegen die Horizontale
! - psi   = Azimuth der Sonne, Sued=0, nach W positiv, nach E negativ
!
! Eingangsgroessen:
! - julia: Julianischer Tag des Jahres (1 bis 365/366)
! - gamma: Sonnenhoehe in dezimalen Grad
! - albedo: kurzwellige Reflexionseigenschaften der Bodens. Ueblicherweise wird
!           entsprechend Klima-Michel 0.2 fuer trockenen, ebenen, kastanien-
!           braunen Boden eingesetzt, was auch Grasland entspricht
! - pb   : Luftdruck an der Station, hPa, (vor Unterprogramm ggf. berech-
!          nen aus Stationshoehe z ueber NN, m: mit pb=1013.25*exp(-z/8434.5))
! - ngrd : Gesamtbedeckungsgrad (0 bis 8/8, 9= Nebel)
! - nh   : Bedeckungsgrad mit tiefen/mittelhohen Wolken
!
! Benoetigte Funktionen:
! - fi0abstk (abstandskorrigierte extraterrestrische Sonnenstrahlung)
! - fdr0m    (vertikale optische Dicke * rel. Luftmasse)
! - tl_julia  leitet den Tagesmittelwert fr den Linke-Truebungsfaktor aus den
!             Monatsmittelwerten im europ. Atlas zur Sonnenstrahlung , Vol.2 ab
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
!
      implicit none
!
!     --- Umrechnungfaktor fuer Grad auf Radians (360 Grad=2*pi) und Initia-
!         lisierung von Konstanten zur Wichtung des Wolkeneinflusses + Albedo
!     ... initialise for conversion degree to radians, barometric pressure 
!         at mean sea level and weighting constants for clouds and albedo 
      real pi,rad,p0,aw,bw
      parameter (pi=3.141593, rad=pi/180., p0=1013.25, aw=0.72, bw=3.2)
!
!     --- Typvereinbarungen fuer die Eingangsgroessen der Argumentliste
!         type of input of the argument list
      integer julia,ngrd,nh
      real gamma,albedo,pb
!
!     --- feste Winkelverhaeltnisse fuer horizontale Flaeche
!         fixed angle ratios for the horizontal surface
      real cosaega,cos2b_2,sin2b_2
!hier auskommentieren
      parameter (cosaega=1.,cos2b_2=1.,sin2b_2=0.)
!hier
!      real alpha,beta,psi
!
!     --- Typvereinbarung fuer die Ausgabegroessen der Argumentliste
!         types of output of argument list
      real difiso,difzrk,dirsnk,dirson_b,g_kw,r_kw
!hier
!      real r_kw_o
!
!     --- Typvereinbarung fuer benoetigte Funktionen
!         types of required functions
      CHARACTER linkepath*(*)
!     real fi0abstk,fdr0m,ftlvdi
!      real fi0abstk,fdr0m,tl_julia
!
!     --- Typvereinbarung fuer das UP
      real s0,s,s_s0,g0,singam,cosaeta,exprm,wokta,tl,pb_p0,zw
      real breite,laenge
      integer nd
!
!
!     --- Diffuse und direkte kurzwellige Strahlungsfluesse initialisieren
!         initialise short-wave radiation fluxes
      difiso  = 0.
      difzrk  = 0.
      dirsnk  = 0.
      dirson_b= 0.
      g_kw    = 0.
      r_kw    = 0.
!hier
!      r_kw_o  = 0.
!
!     --- Sonne unter dem Horizont / sun beneath the horizon
      if(gamma.lt.-1.1) then
         return
      endif
!
!     --- Initialisieren (Bedeckungsgrad in Achteln)
!         Unterschiede hohe und mittelhohe/tiefe Wolken
!         account for differences between cloud cover by low/medium 
!         and high clouds
      nd=0
      if(2.le.ngrd.and.ngrd.le.6) then
         if((ngrd-nh).ge.2) then
            nd=2
         else
            if((ngrd-nh).eq.1) then
               nd=1
            endif
         endif
      endif
      if(0.le.ngrd.and.ngrd.le.8) then
         wokta=float(ngrd-nd)/8.
      else
         if(ngrd.lt.0) then
            wokta=0.
         else
            wokta=1.
         endif
      endif
!     --- Bedeckungsgrad in Achteln (Original VDI 3789/2 ohne Unterschied
!         hohe zu mittelhohen/tiefen Wolken
!      if(0.le.ngrd.and.ngrd.le.8) then
!         wokta=float(ngrd)/8.
!      else
!         if(ngrd.lt.0) then
!            wokta=0.
!         else
!            wokta=1.
!         endif
!      endif
!     --- Sinus der Sonnenhoehe / sine of solar elevation
      if(gamma.ge.0.and.gamma.le.180.) then
         singam=sin(gamma * rad)
      else
         singam=0.
      endif
      cosaeta=singam
!
!     --- Winkelverhaeltnisse der geneigten Flaeche
!         wenn aktiviert, dann "parameter(cosaega=1.,cos2b_2=1.,sin2b_2=0.)"
!         inaktivieren
!hier
!      zw=beta*rad
!      cosaeta=singam*cos(zw)+
!     &        cos(gamma*rad)*sin(zw)*cos((alpha-psi)*rad)
!      if(acos(cosaeta)/rad.gt.90.) then
!         cosaega=0.
!      else
!         cosaega=cosaeta/singam
!      endif
!      zw=beta*rad/2.
!      cos2b_2=(cos(zw))**2.
!      sin2b_2=(sin(zw))**2.
!hier: Ende
!
!     --- Extinktionsfaktor aus Linke Truebungsfaktor, Hoeher ueber NN und
!         rel. optischer Luftmasse
!         extinction using a mean Linke turbidity factor and rel. opt. air mass
      pb_p0=pb/p0
      tl=tl_julia(linkepath,breite,laenge,julia)
      exprm=exp(-1.*tl*pb_p0*fdr0m(gamma))
!
!     --- Direkte Sonne senkrecht zur Einfallsrichtung
!         direct solar irradiation on surface normal to solar elevation
!     --- wolkenlos / clear sky
      s0=fi0abstk(julia)
      s=s0*exprm
!     --- bewoelkt / account for cloudiness
      dirsnk=(1.-wokta)*s
!     --- auf die horizontale/geneigte Flaeche / on a horizontal surface
      dirson_b=dirsnk*cosaeta
!
!     --- Globalstrahlung: Horizontale Flaeche, wolkenloser Himmel
!         global irradiation on a horizontal surface: clear sky
!     VDI eq. 19
      if(singam.ne.0.) then
         g0=0.84*s0*singam*exp(-0.027*pb_p0*tl/singam)
      else
         g0=0.
      endif
      if(g0.lt.0.) g0=0.
!
!     diffuse Strahlung auf die horizontale/geneigte Flaeche / diff irradiation
!     --- anisotroper Teil aus wolkenlosen Himmelsabschnitten (zirkumsolar)
!         anisotropic part from cloudless parts of the sky
      s_s0=s/s0
      zw=(1-wokta)*(g0-s*singam)
      difzrk=zw*s_s0*cosaega
      if(difzrk.lt.0.) difzrk=0.
!     --- isotroper Teil aus wolkenlosen Himmelsabschnitten
!         isotropic part from clouded parts of the sky
      difiso=zw*(1.-s_s0)*cos2b_2
!     --- Summe isotroper Anteil aus wolkenlosen und bewoelkten Anteilen
!         accumulated isotropic and anisotropic diffuse irradiation
      difiso=difiso+wokta*(1.-aw*(wokta**(bw-1.)))*g0*cos2b_2
      if(difiso.lt.0.) difiso=0.
!
!     --- Globalstrahlung bewoelkt auf die horizontale/geneigte Flaeche:
!         nicht "g_kw=g0*(1-aw*(wokta**bw))" verwenden
!         global short-wave irradiation on the horizontal surface
      g_kw=dirson_b+difiso+difzrk
!
!     reflektierte kurzwellige Strahlung, als nur isotrop betrachtet
!     --- nach unten orientierte horizontale Empfangsflaeche
!         reflected isotropic irradiat. on horizontal surface looking downward
      r_kw=albedo*g_kw
!     --- nach oben orientierte horizontale/geneigte Empfangsflaeche
!hier
!      r_kw_o=r_kw*sin2b_2
!
      return
      END subroutine kwstvdi
!
!---------------------------------------------------------------------
      real function fi0abstk(julia)
!---------------------------------------------------------------------
!
! Function: Extraterrestrial solar irradiation corrected for 
! earth sun distance. Input: day number of the year
!
! fi0abstk= Abstandskorrigierte extraterrestrische Sonnenstrahlung, W m-2
!
! Literatur / references: VDI-3782/Blatt 2 (1994)
!
! Der Ausgabewert beruecksichtigt, dass der Umlauf der Erde um die Sonne
! von der Kreisbahn abweicht, so dass die extraterrestrische Sonnenstrah-
! am Oberrand der Atmosphaere einen Jahresgang aufweist
!
! Argumentliste:
! - julia = julianischer Tag des Jahres (max. 365/366)
! Wesentliche Variable:
! - s0 ist die Solarkonstante, W m-2, bei mittlerem Abstand der Erde von der
!      Sonne
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typ des Arguments / type of the arguments
      integer julia
!
!     --- Umrechnungfaktor fuer Grad auf Radians (360 Grad=2*pi) und
!         Solarkonstante 
!         conversion degree to radians and solar constant
      real pi,rad,s0
      parameter (pi=3.141593, rad=pi/180., s0= 1367.)
!
!     --- Funktionswert / value of the function
      fi0abstk=s0*(1.+0.03344*cos(rad*(0.9856*float(julia)-2.72)))
!
      return
      END function fi0abstk
!
!---------------------------------------------------------------------
      real function fdr0m(gamma)
!---------------------------------------------------------------------
!
! Relative optical air mass and vertical optical thickness
!
! Vertikale optische Dicke der Atmosphaere * rel. optischer Luftmasse:
!     - fdr0m=delta(r0)*m
!
! VDI-3789/Blatt 2, Anhang B (1994)
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typ des Argumentes / type of arguments
      real gamma
!     --- Typ der festen Konstanten / type and value of constants
      real a,b,c,pi,rad
      parameter (a=0.50572, b=6.07995, c=1.6364)
      parameter (pi=3.141593, rad=pi/180.)
!
!     --- Typen der Variablen der Funktion / type of programme variables
      real erwrm(11),erwdr0(7),gam,singam,rm,dr0,oben
      integer i
!
!     --- Initialisieren / initialise
      data erwrm/                                                       &
     & 39.22,40.34,41.48,42.63,43.72,44.81,45.9,47.,48.08,49.15,50.12/
      data erwdr0 /0.0548,0.0519,0.0491,0.0463,0.0435,0.0408,0.0381/
!
      gam=gamma
      if(gam.lt.-1.1) gam=-1.1
      singam=sin(gam*rad)
!
!     --- Fall gamma > 10 Grad, Naehrungswert (Fehler < 3 %)
!         case: solar elevation > 10 degree (approximation, error < 3 %)
      if(gam.gt.10.) then
!        --- Funktionswert, Fall gamma >10 Grad / value of the function
         fdr0m=1./(0.9+9.4*singam)
         return
      endif
!
!     --- relative optische Luftmasse / relative optical air mass
      if(gam.ge.0.) then
         rm=1./(singam+a*(gam+b)**(-1.*c))
      else
!        --- Horizonterweiterung / account for extension of the horizon
         do i=1,11
            if(gam.ge.(-0.1*float(i))) then
               rm=erwrm(i)
               goto 200
            endif
         enddo
         rm=erwrm(7)
  200    continue
      endif
!
!     --- vertikale optische Dicke der reinen und trockenen Normatmosphaere,
!         Rayleigh-Atmosphaere
!         vertical optical thicknees of a Rayleigh atmosphere
      if(gam.gt.5.) then
!        --- Funktionswert, Fall gamma > 5 Grad
!            value of the function: case gamma > 5 degree
         fdr0m=1./(0.9+9.4/rm)
         return
      endif
      do i=1,6
         oben=float(6-i)
         if(oben.ge.gam.and.gam.gt.(oben-1.)) then
!           --- lineare Interpolation, Differenz der Schritte delta(gamma)=1
!               linear interpolation: step difference: 1 degree
            dr0=erwdr0(i)-(erwdr0(i)-erwdr0(i+1))*(oben-gam)
            goto 300
         endif
      enddo
      dr0=erwdr0(7)
  300 continue
!
!     --- Funkionswert, Fall gamma <= 5 Grad
!         value of the function: case gamma <= 5 degree
      fdr0m=rm*dr0
!
!     --- Monotonen Abfall der Werte mit zunehmendem Gamma sichern
!         assure a monotonic decrease with increasing solar elevation
      if(gam.lt.0..and.fdr0m.lt.1.54712) then
         fdr0m=1.54712
      endif
!
      return
      END function fdr0m
!
!----------------------------------------------------------------------------------
      REAL function tl_julia(linkepath,rlat,rlon,julia)
!----------------------------------------------------------------------------------
!     The function calculates mean daily turbidity factors by linear interpolation
!     based on the global monthly mean values of the European Solar Radiation
!     Atlas
!----------------------------------------------------------------------------------
!     Required input:
!     ----------------
!     - linkepath: path to the linke map file
!     - rlat : geographical latitude, decimal degree, north:+,south:-; type real
!     - rlon : geographical longitude, decimal degree, north:+,south:-; type real
!     - julia : daynumber of the year; type integer
!
!     subroutine required:
!     --------------------
!     - r_linkemap : reads linke turbidity factor global monthly mean text map files
!
!
!     4.10.2018   rewrote function to improve performance (Stefan Muthers)
!         
!-----------------------------------------------------------------------------------
      implicit none
!
!     --- type declaration linke map
      character :: linkepath*(*)
      
!     ... defines the array dimensions: lon/lat
      integer, parameter :: nlzea=1,nlze=2160,nlspa=1,nlsp=4320
      character (len=1)  ::  c_linke(nlspa:nlsp,nlzea:nlze)
!
!     ... defines the geograhpic grid 
      real     ::  rl_west,rl_south,h_step
      parameter(h_step=1./24.,rl_west=-180.+h_step,rl_south=-90.+h_step)
!
!     --- Typvereinbarung fuer Zielkoordinaten Stationsliste TOMS
      real      :: zielkb,zielkl
      integer   :: j_s,i_s
!
!
!     --- types of logical unit
      integer   :: luin=20
!
!     --- types of programme variables
!
      character :: linkefile*100
      integer   :: ix,mm,me,monat(12),im
      integer   :: julia
      real      :: rlat,rlon,gew,tl1,tl2
      integer   :: thismonth, thisday, doytmp
!
      data monat /31,28,31,30,31,30,31,31,30,31,30,31/
!
!----------------------------------------------------------------------------------
      linkefile=' '
      linkefile(1:)='TL5_MM.raw'
      ix=index(linkefile,'MM')
!
      zielkb=rlat
      zielkl=rlon
      j_s=nlze-int((zielkb-rl_south)*12.)
      i_s=int((zielkl-rl_west)*12.) +1

      if(julia.EQ.366) julia= 365

      ! Monat und Tag aus DOY berechnen
        doytmp = 0
      do mm=1,12
            if(julia.le.(doytmp+monat(mm))) then
                thismonth = mm
                thisday = julia - doytmp 
                exit
            end if
            doytmp = doytmp +  monat(mm)
      end do
        !write(*,*) 'Doy: ', julia, ' Datum: ', thisday, '.', thismonth 


        ! get linke factor for the current months
      write(linkefile(ix:ix+1),fmt='(1i2.2)') thismonth
      call r_linkemap(luin,linkepath,linkefile,c_linke,              &
                     & nlspa,nlsp,nlzea,nlze)
      tl1 = real(ichar(c_linke(i_s,j_s)))/20.

      gew = 0.               
        tl2 = 0
        if(thisday.le.6) then
                  ! gewichtetes Mittel mit Vormonat
              im=mm-1
              if(im.le.0) im=12
                  write(linkefile(ix:ix+1),fmt='(1i2.2)') im
                  call r_linkemap(luin,linkepath,linkefile,c_linke,              &
                                      & nlspa,nlsp,nlzea,nlze)
                  tl2 = real(ichar(c_linke(i_s,j_s)))/20.
              gew = (6.-float(thisday))/11.
        end if
        me = monat(mm)-5
        if(thisday.ge.me) then
                  ! gewichtetes Mittel mit Folgemonat
              im=mm+1
              if(im.gt.12) im=1
                  write(linkefile(ix:ix+1),fmt='(1i2.2)') im
                  call r_linkemap(luin,linkepath,linkefile,c_linke,              &
                                      & nlspa,nlsp,nlzea,nlze)
                  tl2 = real(ichar(c_linke(i_s,j_s)))/20.
              gew = (float(thisday)-real(me))/11.
        end if
      tl_julia = tl1 + gew * (tl2-tl1)

      end function tl_julia

!
!-----------------------------------------------------------------------
      subroutine r_linkemap(luin,linkepath,linkefile,c_linke,           &
     &                      nlspa,nlsp,nlzea,nlze)
!-----------------------------------------------------------------------
!
! Reads Linke Turbidity Factor global monthly mean text map files
! 
! 
! Regular geographic grid:
! - 1 to 4320 pixel in longitudinal direction: -179.95833 (W) to +179.95833 (E)
! - 1 to 2160 pixel in latitudinal  direction:  +89.95833 (N) to  -89.95833 (S)
! - Pixel distance in latitudinal and longitudinal direction: 1/12 degree
!
! Argumentlist:
! -------------
! - luin     : logical unit for input, type integer
! - linkepath: path to the linke map file
! - linkefile: filename of the linke map file TL5_MM.raw of month MM
! - c_linke(nlspa:nlsp,nlzea:nlze), type character*1
!     content of the linke map file:
!   = nlspa,nlsp : index longitude (columns): 1(1)4320
!   = nlzea,nlze : index latitude  (rows)   : 1(1)2160
!
! References:
! -----------
! Scharmer K, Greif J (ed.) (2000): The European Solar Radiation Atlas,
!     Vol. 2: Database and Exploitation Software. Les Presses de l'Ecole
!     des Mines, Paris. ISBN: 2-911762-22-3
! - Remund J, Wald L, Lefevre M, Ranchin T (2003): Worldwide Linke Turbidity
!     information. Proceedings of ISES Solar World Congress, 16-19 June 2003,
!     Goeteborg, Sweden, pp. 13.
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- type declaration linke map
      character :: linkepath*(*),linkefile*(*)
!     ... defines the array dimensions: lon/lat
      integer :: nlzea,nlze,nlspa,nlsp
!      integer, parameter :: nlzea=1,nlze=2160,nlspa=1,nlsp=4320
      character (len=1) :: c_linke(nlspa:nlsp,nlzea:nlze)
!
!     ... defines the geograhpic grid 
!      real rl_west,rl_south,srl_lang,srl_breit,h_step
!      parameter(h_step=1./24.,rl_west=-180.+h_step,rl_south=-90.+h_step)
!      parameter(srl_lang=2.*h_step,srl_breit=2.*h_step)
!
!     --- types of the argumentlist
      integer :: luin
!
!     --- type of programme variables
      integer :: iocheck
!
!
      open(unit=luin,file=linkepath(1:len_trim(linkepath))//            &
     &     linkefile(1:len_trim(linkefile)),status='old',               &
     &     access='direct',form='unformatted',                          &
     &     recl=(nlsp-nlspa+1)*(nlze-nlzea+1),iostat=iocheck)
      if(iocheck.ne.0) then
         write(*,*) 'Fehler, file= ',linkepath(1:len_trim(linkepath))//   &
     &     linkefile(1:len_trim(linkefile))
         write(*,*) 'iostat= ',iocheck
         stop
      else
!        write(*,*) 'offen, file= ',linkepath(1:len_trim(linkepath))//    &
!    &     linkefile(1:len_trim(linkefile))
      endif
!
      read (luin,rec=1) c_linke
!
      close(unit=luin)
!
      return
      end subroutine r_linkemap
!

!---------------------------------------------------------------------
      subroutine lwstvdi(tt,vau1,qk,em,lndntzg,xnl,xnm,xnh,             &
     &                   tbc,a_lw,e_lw)
!---------------------------------------------------------------------
!
! LWSTVDI.FOR:
!
! Long-wave irradiation on a horizontal surface (down- and up-welling)
! - down-welling component according to VDI 3789/2
! - up-welling component according to Jendritzky et al. based on Bowen ratio
!
! Input:
! ------
! - tt     : air temperature, degree C, 2 m above ground
! - vau1   : air velocity 1 m above ground, m/s
! - qk     : short-wave radiation balance on the horizontal surface, W m-2
! - em     : long-wave emissivity of the ground (often 0.95)
! - lndntzg: land use: 1= open land, 2= settlement, 3= water)
! - xnl    : cloud cover by low clouds   : 0.0 to 1.0
! - xnm    : cloud cover by medium clouds: 0.0 to 1.0
! - xnh    : cloud cover by high clouds  : 0.0 to 1.0
!
! Output:
! -------
! - tbc    : temperature of the ground, degree C
! - a_lw   : down-welling long-wave irradiation on a horizontal surface, W/m2
! - e_lw   : up-welling long-wave irradiation on a horizontal surface
!            including the reflected part down-welling irradiation, W/m2
!
! Langwellige Strahlung der Atmosphaere auf die horizontale Empfangsflaeche:
! VDI 3789/2 - Modell zur parametrisierten Berechnung der
!              langwelligen Strahlungsfluesse der Atmosphaere
!
! Literatur / references: 
!            VDI (1994): Umweltmeteorologie, Wechselwirkungen zwischen
!            Atmosphaere und Oberflaechen. Berechnung der kurz- und lang-
!            welligen Strahlung. VDI-Richtlinie 3789, Blatt 2
!
!            Kasten F (1989): Strahlungsaustausch zwischen Oberflaechen
!            und Atmosphaere. VDI-Berichte
!
!            Jendritzky G et al. (1990): Methodik zur raeumlichen Bewertung
!            der thermischen Komponente im Bioklima des Menschen. ARL Bei-
!            traege 114.
!
! Langwellige Strahlung des Erdbodens:
! Fuer die Parametrisierung der langwelligen Strahlung der Erdoberflaeche
! wird die Parametrisierung aus dem Klima-Michel (LWSTJEN.FOR) entnommen,
! da die Temperatur der Erdbodenoberflaeche in Synop-Meldungen nicht zur
! Verfuegung steht. Die vom Erdboden reflektierte langwellige Strahlung der
! Atmosphaere wird zur langwelligen Strahlung des Erdbodens addiert.
!
!
! Ausgabegroessen:
! - tbc    : Temperatur der Erdoberflaeche in Grad Celsius
!            Falls gemessen, kann e_lw ueber Messwert bestimmt werden
! - a_lw   : langwellige Waermestrahlung der Atmosphaere, W m-2
! - e_lw   : langwellige Waermestrahlung der Erdoberflaeche + reflektierter
!            Anteil der Waermestrahlung der Atmosphaere, W m-2
!
! Eingangsgroessen der Argumentliste
! - tt     : Lufttemperatur, Grad C, 2 m ueber Grund
! - vau1   : Windgeschwindigkeit 1 m ueber Grund, m/s
! - qk     : Strahlungbilanz der horizontalen Flaeche (kurzwellig), W m-2
! - em     : langwellige Emissionszahl des Erdbodens (meist 0.95)
! - lndntzg: Landnutzung: 1= Freiland, 2= Siedlung, 3= Wasser)
! - xnl    : Bedeckung mit tiefen Wolken: 0.0 bis 1.0
! - xnm    : Bedeckung mit mittelhohen Wolken: 0.0 bis 1.0
! - xnh    : Bedeckung mit hohen Wolken: 0.0 bis 1.0
!
! Benoetigte Funktionen / required functions:
! - tboden (Jendritzky): Fall: Erdbodentemperatur nicht gemessen
!                        case: temperature of ground surface not measured
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Konstanten fuer Stefan-Boltzmann und effektiven
!         halbraeumigen Emissionsgrad der wolkenlosen Atmosphaere
!         constants: Stefan-Boltzmann, absolute temperature, emissivity
!                    of a cloudless atmosphere
      real kelvin,sigma,epseff
      parameter (kelvin=273.2,sigma=5.67E-8,epseff=9.9E-6)
!
!     --- Typvereinbarung fuer Eingangsgroessen der Argumentliste
!         type of input of argumentlist
      real tt,vau1,qk,em,xnl,xnm,xnh
      integer lndntzg
!
!     --- Typvereinbarung fuer Ausgabegroessen der Argumentliste
!         type of output of argumentlist
      real tbc,a_lw,e_lw
!
!     --- Typvereinbarungen fuer Variablen des Programms 
!         type of programme variables
      real tstr4,ttwb,tk,r_lw
      real al,am,ah,a0
!
!     --- Typvereinbarrung fuer benoetigte Funktionen / type of functions
!      real tboden
!
!     --- Initialisieren / initialise
      tk = tt + kelvin
      al=2.30-7.37e-3*tk
      am=2.48-8.23e-3*tk
      ah=2.89-1.00e-2*tk
!
!     --- Gegenstrahlung der wolkenfreien Atmosphaere
!         down-welling long-wave irradiation of the cloudless atmosphere
      a0=epseff*sigma*tk**6.
!
!     --- Gegenstrahlung der bewoelkten Atmosphaere
!         down-welling long-wave irradiation of the clouded atmosphere
      a_lw=a0*(1.+al*xnl**2.5+(1.-xnl)*am*xnm**2.5+                     &
     &        (1.-xnl)*(1.-xnm)*ah*xnh**2.5)
!
!     --- vom Erdboden reflektierter langwelliger Anteil der Gegenstrahlung
!         reflected long-wave irradiation by the ground
      r_lw=(1.-em)*a_lw
!
!     --- Berechnung der Oberflaechentemperatur des Bodens (Klima-Michel)
!         calculate surface temperature of the ground 
      tstr4= a_lw/sigma
      ttwb = tboden(tk, vau1, qk, tstr4, lndntzg, em)
      tbc  = ttwb-kelvin
!
!     --- langwellige Waermestrahlung des Erdbodens, W m-2 + reflekt. Gegenst.
!         up-welling long-wave irradiation from the ground including reflected
!         parts
      e_lw = em * ttwb **4 * sigma + r_lw
!
      return
      END subroutine lwstvdi
!
!---------------------------------------------------------------------
      real FUNCTION tboden (tk, vau1, qk, tstr4, lndntzg, em)
!---------------------------------------------------------------------
!
! The function calculates the temperature of the ground surface (K)
! accounting for land use (lndntzg) according to the Bowen ratio for 
! 1= open land, 2= settlement, 3= water, tk = air temperature, qk = 
! short-wave radiation balance, tstr4= down-welling long-wave irradition 
! divided by the Stefan-Boltzmann constant, vau1= wind velocity (m/s) 1 m 
! above ground, and emissivity of the ground.
!
!     Die Function berechnet die Oberflaechentemperatur des Bodens (Kelvin)
!     unter Beruecksichtigung der Landnutzung bzgl. Bowen-Verhaeltnis mit
!     lndntzg 1 Forst- und Landwirtschaft, 2 Siedlung, 3 Wasser.
!     Bestimmung des Bodenwaermestroms (boden) anhand der kurzwelligen
!     Strahlungsbilanz Qk: ib= 1 fuer QK > 0., sonst (2).
!     Bowen 1 fuer Forst- und Landwirtschaft, 2 Siedlung, 3 Wasser.
!     ALPHAL nach Suppan Formel 1.18 S.8.
!     Die Rueckrechnung von vau1 (1 m Hoehe) auf vau10 (10 m Hoehe) wurde in
!     Absprache mit Dr. Jendritzky vom 16.08.93 von bisher Faktor 2. auf
!     Faktor 1.5 geaendert; dadurch Konsistenz zur eigentlichen Messung
!
      implicit none
!
      real sigma
      parameter (sigma=5.67E-8)
      real  boden (2), bowen (3), tk, vau1, vau10, qk, tstr4, em,       &
     &      alphal, zaehler, nenner
      integer ib, lndntzg, i
      DATA  boden /-.19, -.32/, bowen /.8, 1.3, .1/
!
!     --- Rueckrechnung Wind in 1 m Hoehe [m/s] auf Wind in 10 m Hoehe [m/s]
!         back-calculation to wind velocity 10 m above ground
      vau10 = 3./2. * vau1
!
      alphal = 6.2 + 4.26 * vau10
!
      IF(qk .LE. 0.) THEN
         ib = 2
      ELSE
         ib = 1
      ENDIF
!
!     --- Newton'sche Naeherung zur Berechnung der Oberflaechentemperatur
!         des Bodens mit Lufttemperatur tk [K] als nullter Naeherung
!         Newton approximation of ground surface temperature, K
      tboden = tk
      DO  i = 1, 3
         zaehler = (qk + em * sigma * (tstr4 - tboden ** 4)) * (1. +    &
     &             boden(ib)) + (1. + 1. / bowen(lndntzg)) * alphal *   &
     &             (tk - tboden)
         nenner = -4. * em * sigma * (1. + boden(ib)) * tboden ** 3 -   &
     &            (1. + 1. / bowen(lndntzg)) * alphal
         tboden = tboden - zaehler / nenner
      ENDDO
!
      return
      END function tboden
!
!---------------------------------------------------------------------
      real function ftmrt_g(gamma,gh_kw,dh_kw,r_kw,a_lw,e_lw)
!---------------------------------------------------------------------
!
! FTMRT_G.FOR
!
! The value of the function FTMRT_G is the mean radiant temperature, degree
! centigrade, effective for an upright standing person. The environment is a
! plain: solid angles of upper and lower half-space are wifak=0.5.
!
!
! Input from argument list (types are real):
! ------------------------------------------
! - gamma  : solar altitude angle, degree: type real
! - gh_kw  : global solar radiation on a horizontal surface
!            (short-wave), W m-2: type real
! - dh_kw  : diffuse solar radiation on a horizontal surface
!            (short-wave), W m-2: type real.
!            If availabe use only the isotropic part of diffuse radiation.
! - r_kw   : reflected global radiation
!            (short-wave), W m-2: type real
! - a_lw   : thermal radiation of the athmosphere
!            (long-wave), W m-2: type real
! - e_lw   : thermal radiation of the earth surface
!            (long-wave), W m-2: type real
!
! Function value:
! ---------------
! - ftmrt_g: mean radiant temperature, degree centigrade: type real
!
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! source: Klima Michel by Prof. Dr. G. Jendritzky
! code by Dr. Henning Staiger
!
!*********************************************************************
!
!
      implicit none
!
!     --- Umrechnungfaktor fuer Grad auf Radians (360 = 2*pi)
!         conversion of degree to radians
      real pi,rad
      parameter (pi=3.141593, rad=pi/180.)
!
!     --- Konstanten fuer Stefan-Boltzmann etc. / required constants
      real kelvin,sigma
      parameter (kelvin=273.2, sigma=5.67E-8)
!
!     --- Konstanten fuer Berechnung tmrt aus Strahlungstemperaturen
!         wifak=Winkelfaktor fuer Halbraum, alphir=Absorptionskoeffizient
!         der Bekleidung, ep= Emissionzahl der Person (Fanger, Gl. 51)
!         constants to calculate the mean radiant temperature:
!         wifak= solid angle component for down- and up-welling irradiation
!         alphir= Fanger's short-wave absorption coefficient of skin and 
!         clothing, ep = lng-wave emissivity of clothing
      real wifak,alphir,ep,c
      parameter (wifak=0.5, alphir=0.7, ep=0.97, c=1./ep/sigma)
!
!     --- Typvereinbarung fuer Eingangsgroessen der Argumentliste
!         type of input of the argument list
      real gamma, gh_kw, dh_kw, r_kw, a_lw, e_lw
!
!     --- Typvereinbarungen fuer Variable des Programms
!         types of programme variables
      real fkts4, fkth4, fktb4, tsta4, tste4, tsgeg4, fp, singam
      real tumrt4, tumrt, tmrt4, dirsnk
!
!     --- Sinus der Sonnenhoehe initialisieren / sine of solar elevation
      if(abs(gamma).gt.1.) then
         singam=sin(gamma*rad)
      else
         singam=0.
      endif
!
!     --- direkte Strahlung senkrecht zur Einfallsrichtung
!         direct irradiation on a surface normal to solar elevation
      if(gamma.gt.0..and.singam.gt.0.) then
         dirsnk=(gh_kw-dh_kw)/singam
         if(dirsnk.lt.0.) dirsnk=0.
      else
         dirsnk=0.
      endif
!
!     --- Strahlungstemperatur der Waermestrahlung der Atmosphaere, K**4
!         radiant temperature of the atmosphere, K**4
      tsta4=a_lw/sigma
!
!     --- Strahlungstemperatur aus Waermestrahlung des Erdbodens, K**4
!         radiant temperature of the ground, K**4
      tste4=e_lw/sigma
!
!     --- langwellige Anteile an tmrt / long-wave component of tmrt
      tsgeg4= wifak * (tsta4 + tste4)
!
!     ---  kurzwellige Anteile an tmrt/ short-wave component of tmrt
      IF(singam .GT. 0.)  THEN
!        --- Projected Area Factor
         fp = .308 * COS (rad * gamma * (.998 - gamma **2 / 50000.))
!        --- direkter Strahlungsanteil / component by direct irradiation
         fkts4 = fp * alphir * dirsnk * c
!        --- diffuse Strahlungsanteile / component by diffuse irradiation
         fkth4 = wifak * alphir * dh_kw * c
         fktb4 = wifak * alphir * r_kw * c
      ELSE
         fkts4 = 0.
         fkth4 = 0.
         fktb4 = 0.
      END IF
      tumrt4 = fkth4 + fktb4 + tsgeg4
      tumrt  = SQRT(SQRT(tumrt4)) - kelvin
      tmrt4  = tumrt4 + fkts4
!
!     --- Funktionswert / value of the function
      ftmrt_g  = SQRT(SQRT(tmrt4)) - kelvin
!
      return
      END function ftmrt_g
!
!---------------------------------------------------------------------
  !
!-----------------------------------------------------------------------
      real function pt_basic(tt2m,el2m,tmrt,vau1m,pb,                   &
     &                       pmva,clo,nerr,ptc,schwuel,d_std,pmvs)
!-----------------------------------------------------------------------
!
!     PT_BASIC.F90   Version of perceived temperature (PT, C) for
!     - standard measured/predicted meteorological values and TMRT 
!       as input;
!     - regressions for determination of PT;
!     - adjustment to Gagge's PMV* (2-node-model, 1986) as base of PT 
!       under warm/humid conditions (Icl= 0.50 clo) and under cold 
!       conditions (Icl= 1.75 clo)
!
! Function value is the Perceived Temperature, degree centigrade
!
! Definition:
! -----------
! The Perceived Temperature is the air temperature in a reference environ-
! ment that results in the same Predicted Mean Vote (Fanger 1972), PMV, as 
! the real environment. In the reference environment the mean radiant 
! temperature is equal to the air temperature and the wind speed is reduced 
! to a value set at 0.1 m/s. The Perceived Temperature is linked to a 
! reference individual: male aged 35 years, body height 1.75 m, weight
! 75 kg, walking with 4 km per hour on a horizontal plain (related to an
! internal heat production of 172.5 W). The person varies the thermal
! resistance of clothing (clo) in the range Icl=1.75 clo (winter) and 
! Icl=0.50 clo (summer) to achieve thermal comfort (PMV = 0) if possible. 
! If not possible the individual perceives cold stress (winter, 1.75 clo) 
! or heat load (summer, 0.5 clo).
!
! Input arguments:
! ----------------
! - tt2m       : air temperature, degree centigrade, 2 meter above ground
!                level: type real
! - el2m       : water vapour presure, hPa=hecto Pascal, 2 meter above ground
!                level: type real
! - tmrt       : mean radiant temperature, degree centigrade, 2 meter above
!                ground level, see Function FTMRT.F: type real
! - vau1m      : Wind speed in 1 m above ground level, unit m/s: type real
! - pb         : barometric pressure at site altitude , unit hPa: type real
!
! Function value and output arguments:
! ------------------------------------
! - pt_basic: perceived temperature, degree centigrade: type real
! - pmva    : Predicted Mean Vote according to Fanger (1972) after adjustment
!             of clothing in the range clo=1.75 (winter) and clo=0.50 (summer)
!             to achieve thermal comfort if possible: type real
! - clo     : clothing insulation (clo), varying between the limits clo=0.5
!             (warm/hot conditions) and clo=1.75 (cold conditions): type real
! - nerr    : Status of function pt_basic, type integer: 
!             =0 : o.k.;
!             >0 : o.k., function iso: number of iteration steps to convergence
!             keep attention: value of function may be doubtful
!             -1 : function iso_Ridder:
!                  - singularity in evaluated function, or
!                  - no convergence within 15 steps of iteration, or
!                  - root not bracketed by lower and upper threshold of clo;
!             information only, value of function o.k.
!             -2 : function deltapmv: pmva outside of valid regression range;
!             -3 : function deltapmv: rel. humidity set to 5 % or 95 %, resp.
!             -4 : function deltapmv: pmvs originally < 0, set: 0 to +0.11;
!             -5 : function delta_kalt: intersection points of regression lines
!                  undefined.
! - pmvs=   : rational PMV* according to Gagge 1986, hc= Fanger
! - d_pmv   : increment to pmva to calculate PMV*
! - ptc     : classical perceived temperature (not adjusted to humid and/or 
!             cold conditions
! - schwuel : character*1= 's' -> condition of sultriness. ' ' -> no sultriness
! - d_std   : ratio standard deviation in real humid/warm environment normalised 
!             to standard deviation in mean pt versus ptc
!
! Method:
! -------
! - Thermal comfort equation of Fanger (1972) to calcuate the Predicted Mean
!   Vote (PMV) in the actual environment (pmva).
! - Iteration of clothing thermal resistance (clo) by the procedure of Ridder 
!   to achieve thermal comfort (pmva=0) in the real environment if possible,
!   function ISO_RIDDER.
! - If not possible: Fixed clo (cold conditions Icl=1.75, warm/hot conditions
!   Icl=0.50 clo) and calculation of rational PMV* <> 0 in the real environment. 
!   PMV* according to Gagge et al. (1986) replaces Fanger's PMV
! - Calculating Perceived Temperature by means of regression with the
!   independent variables pmvs and clo.
!
! Required functions and subroutines:
! -----------------------------------
! - subroutine fanger     (Fanger's Predicted Mean Vote: PMV)
! - function iso_Ridder   (clothing insulation required to achieve PMV=0, comfort)
! - function gtregr       (regressions PT dependent on PMV and clothing insulation)
! - function deltapmv     (heat load: warm/humid conditions: adjust to Gagge's PMV*)
! - function gschwuel     (sultriness, acclimatised to Central European conditions)
! - function dpmv_kalt    (cold stress: cold conditions: adjust to Gagge's PMV*)
! - function dpmv_adj     (called by dpmv_kalt)
! - function dampfdruck   (saturation water vapour pressure, hPa)
! - function ireq_neutral (required clothing insulation for thermal neutrality)
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- type of input of the argument list
      real      :: tt2m,el2m,tmrt,vau1m,pb
!
!     --- type of output of the argument list
      character :: schwuel*1
      real      :: pmva,clo,ptc,d_std,pmvs,top
      integer   :: nerr,nerr_kalt
!
!     --- type and set value of subject and activity data 
!         defines Klima-Michel instead calling PERSDAT
      integer,parameter:: sex=1
!     --- Parameter fuer Standard-Klima-Michel (ersetzt Intialisierung und
!         Aufruf von UP PERSDAT)
      real,parameter:: alt=35.,gew=75.,gr=1.75,arb=172.5,eta=0.,        &
     &             adu=1.908094,energieumsatz=256.9939,actlev=134.6862
!
!     --- type of required functions
!      real      :: iso_Ridder,gtregr,deltapmv,dampfdruck,gschwuel,      &
!     &             ireq_neutral,dpmv_kalt
!
!     --- type of program variables
      real,parameter:: eps=0.0005
      real      :: uclo,oclo,tt2mx,tmrtx,el2mx,vau1x,d_pmv,svp_tt,      &
     &             grenz_schw,dgtcm,dgtcstd,clon,ireq_minimal,          &
     &             clo_fanger,pmv_o,pmv_u
      integer   :: nzaehl
!
!     --- initialise
      nerr=0
      nzaehl=0
!     ... tresholds: clothing insulation (account for model inaccuracies)
!     ... summer 
      uclo=0.44453
!     ... winter
      oclo=1.76267
!     ... meteorolgical variables
      vau1x=vau1m
      tt2mx=tt2m
      tmrtx=tmrt
      el2mx=el2m
!
!     --- decision: firstly calculate for winter or summer clothing
      if(tt2m.gt.10.) goto 300
!
!     --- first guess: winter clothing insulation: cold stress
      clo=oclo
      call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,pmva,top)
      pmv_o=pmva
!
      if(pmva.gt.0.) then
!
!        --- case summer clothing insulation: heat load ?            
         clo=uclo
         call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,pmva,top)
         pmv_u=pmva
         if(pmva.le.0.) then
!           --- case: comfort achievable by varying clothing insulation 
!                     between winter and summer set values
            clo=iso_Ridder(tt2mx,tmrtx,el2mx,vau1x,pb,actlev,eta,       &
     &                     uclo,pmv_u,oclo,pmv_o,eps,pmva,top,nzaehl)
            if(nzaehl.lt.0) then
               nerr=-1
               goto 600
            endif
         else if(pmva.gt.0.06) then
            clo=0.5
            call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,      &
     &                  pmva,top)
         endif
      else if(pmva.lt.-0.11) then
         clo=1.75
         call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,pmva,top)
      endif
      goto 500
!
  300 continue
!
!     --- first guess: summer clothing insulation: heat load
      clo=uclo
      call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,pmva,top)
      pmv_u=pmva
!
      if(pmva.lt.0.) then
!
!        --- case winter clothing insulation: cold stress ?
         clo=oclo
         call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,pmva,top)
         pmv_o=pmva
!
         if(pmva.ge.0.) then
!
!           --- case: comfort achievable by varying clothing insulation 
!                     between winter and summer set values
            clo=iso_Ridder(tt2mx,tmrtx,el2mx,vau1x,pb,actlev,eta,       &
     &                     uclo,pmv_u,oclo,pmv_o,eps,pmva,top,nzaehl)
            if(nzaehl.lt.0) then
               nerr=-1
               goto 600
            endif
         else if(pmva.lt.-0.11) then
            clo=1.75
            call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,      &
     &                  pmva,top)
         endif
      else if(pmva.gt.0.06) then
         clo=0.5
         call fanger(tt2mx,tmrtx,el2mx,vau1x,pb,clo,actlev,eta,pmva,top)
      endif
!
  500 continue
!
!     --- determine perceived temperature by regression equation + adjustments
      pmvs=pmva
      pt_basic=gtregr(pmva,clo)
      ptc=pt_basic
      if(clo.ge.1.75.and.pmva.le.-0.11) then
!        ... adjust for cold conditions according to Gagge 1986
         d_pmv=dpmv_kalt(pmva,tt2mx,vau1x,tmrtx,nerr_kalt)
         if(nerr_kalt.gt.0) nerr=-5
         pmvs=pmva-d_pmv
         if(pmvs.gt.-0.11) then
            d_pmv=0.
            pmvs=-0.11
         endif
         pt_basic=gtregr(pmvs,clo)
      endif
      clo_fanger=clo
      clon=clo
      if(clo.gt.0.5.and.pt_basic.le.8.73) then
!        ... required clothing insulation (ireq) is exclusively defined for 
!            operative temperatures (top) less 10 (C) for a 
!            reference wind of 0.2 m/s according to 8.73 (C) for 0.1 m/s
         clon=ireq_neutral(pt_basic,ireq_minimal,nerr)
         clo=clon
      endif
      grenz_schw=gschwuel(ptc,dgtcm,dgtcstd)
      schwuel=' '
      d_std=-99.
!      if(pmva.gt.0..and.clo.le.0.5) then
      if(pmva.gt.0.06.and.clo.le.0.5) then
!        ... adjust for warm/humid conditions according to Gagge 1986
         svp_tt=dampfdruck(tt2mx)
         d_pmv=deltapmv(pmva,tt2mx,el2mx,svp_tt,tmrtx,vau1x,nerr)
         pmvs=pmva+d_pmv
         pt_basic=gtregr(pmvs,clo)
         if(grenz_schw.lt.99.) then
            if((pt_basic-ptc).gt.grenz_schw) schwuel='s'
!           ... d_std= ptc in Relation zur 1Sigma-Schwuelegrenze
!               d_std= factor to threshold for sultriness
            if(abs(dgtcstd).gt.0) then
               d_std=((pt_basic-ptc)-dgtcm)/dgtcstd
            endif
         endif
      endif
!
  600 continue
      if(nerr.eq.0) nerr=nerr+nzaehl
      return
      END function pt_basic
!
!-----------------------------------------------------------------------
      real function iso_Ridder(tt2m,tmrt,el2m,vau1m,pb,actlev,eta,      &
     &                         uclo,pmv_u,oclo,pmv_o,eps,pmva,top,nerr)
!-----------------------------------------------------------------------
!
! Find for actual meteorological conditions (tt2m,tmrt, el2m, vau1m, 
! pb) and set values of individual's activity level (Klima-Michel: 
! 134.682 W/m2 = 2.3 met, mechanincal work load eta= 0 W/m2) that 
! clothing insulation value (clo), where Fanger's Predicted Mean Vote 
! (PMV) equals comfort (pmva=0), i.e. find the root of Fanger's comfort 
! equation) = value of function iso_Ridder.
!
! Input variables of argument list:
! ---------------------------------
! - tt2m  = ambient temperature (C) at screen level
! - tmrt  = mean radiant temperature (C) at screen level
! - el2m  = water vapour pressure (hPa) at screen level
! - vau1m = wind speed (m/s) 1 m above ground
! - pb    = barometric pressure (hPa) at site (equilvalent to site altitude)
! - actlev= individuals activity level per unit surface area (W/m2)
! - eta   = individuals mechanical work load (W/m2)
! - uclo  = lower threshold of bracketing clothing insulation (clo) 
! - pmv_u = Fanger's PMV corresponding to uclo
! - oclo  = upper threshold of bracketing clothing insulation (clo)
! - pmv_o = Fanger's PMV corresponding to oclo
! - eps   = (0.05) accuracy in clothing insulation (clo) for 
!           evaluation the root of Fanger's PMV (pmva=0)
!
! Output variables of argument list:
! ----------------------------------
! - pmva  = 0 (set at zero, because clo is evaluated for comfort)
! - top   = operative temperature (C) at found root of Fanger's PMV
! - nerr  = error status.
!           nerr >= 0, o.k., and nerr is the number of iterations for
!                              convergence
!           nerr = -1: error = mal function of Ridder's convergence method
!           nerr = -2: error = maximum iterations (max_iteration) exceeded 
!           nerr = -3: error = root not bracketed between uclo and oclo
!
! Method:
! -------
! The root is located using Ridder's more powerful variant (Press et al. 
! 1996) on false position (regula falsi). The method suppose that the 
! root is bracketed by a given lower (uclo) and upper (oclo) threshold. 
! Convergence is superlinear (m ~ 1.414).
!
! Reference:
! ----------
! Press WH, Teukolsky SA, Vetterling WT, Flannery BP (1996): Numerical 
!      Receipes in Fortran 77, The Art of Scientific Computing, Second 
!      Edition. Cambridge University Press, pp. 933.
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     ---- types of argument list
      real      :: tt2m,tmrt,el2m,vau1m,pb,actlev,eta,uclo,oclo,eps,    &
     &             pmv_o,pmv_u,pmva,top
      integer   :: nerr
!
!     --- type of program variables
      integer,parameter:: max_iteration=15
      real,   parameter:: guess_0=-1.11e30
      real      :: uclox,oclox,clox,pmvx
      real      :: x_ridder,clo_u,clo_o,x_unten,x_oben,x_mittel,x_neu,  &
     &             y_unten,y_oben,y_mittel,y_neu,s
      integer   :: j
!
!     --- initialise
      nerr=0
!     ... set pmva=0 (comfort): Root of PMV depending on clothing insulation
      pmva=0.
      clo_u=uclo
      y_unten=pmv_u
      clo_o=oclo
      y_oben=pmv_o
      if((y_unten.gt.0..and.y_oben.lt.0.).or.                           &
     &   (y_unten.lt.0..and.y_oben.gt.0.) ) then
         x_unten=clo_u
         x_oben=clo_o
         x_ridder=guess_0
         do j=1,max_iteration
            x_mittel=0.5*(x_unten+x_oben)
            call fanger(tt2m,tmrt,el2m,vau1m,pb,x_mittel,actlev,eta,    &
     &                  y_mittel,top)
!            pmvx=y_mittel
            s=sqrt(y_mittel**2-y_unten*y_oben)
            if(s.eq.0.) then
               iso_Ridder=x_mittel
               nerr=j
               return
            endif
            x_neu=x_mittel+(x_mittel-x_unten) *                         &
     &                     (sign(1.,y_unten-y_oben)*y_mittel/s)
            if(abs(x_neu-x_ridder).le.eps) then
               iso_Ridder=x_ridder
               nerr=j
               return
            endif
            x_ridder=x_neu
            call fanger(tt2m,tmrt,el2m,vau1m,pb,x_ridder,actlev,eta,    &
     &                  y_neu,top)
!            pmvx=y_neu
            if(y_neu.eq.0.) then
               iso_Ridder=x_ridder
               nerr=j
               return
            endif
            if(sign(y_mittel,y_neu).ne.y_mittel) then
               x_unten=x_mittel
               y_unten=y_mittel
               x_oben=x_ridder
               y_oben=y_neu
            else if(sign(y_unten,y_neu).ne.y_unten) then
               x_oben=x_ridder
               y_oben=y_neu
            else if(sign(y_oben,y_neu).ne.y_oben) then
               x_unten=x_ridder
               y_unten=y_neu
            else
!              ... never get here in x_ridder: singularity in y
               nerr=-1
               iso_Ridder=x_ridder
               return
            endif
            if(abs(x_oben-x_unten).le.eps) then
               iso_Ridder=x_ridder
               nerr=j
               return
            endif
         enddo
!        ... x_ridder exceed maximum iterations
         nerr=-2
         iso_Ridder=y_neu
         return
      else if(y_unten.eq.0.) then
         x_ridder=clo_u
      else if(y_oben.eq.0.) then
         x_ridder=clo_o
      else
!        ... x_ridder not bracketed by u_clo and o_clo
         nerr=-3
         iso_Ridder=x_ridder
         return
      endif
!
      return
      end function iso_Ridder
!
!---------------------------------------------------------------------
      real function gtregr(pmv,clo)
!---------------------------------------------------------------------
!
! Regression relations between perceived temperature (gt) and (adjusted) 
! PMV (21.11.1996, files F3PMVPGT.TXT und F3CLOGT.TXT). Three cases:
!
! - PMV < 0: gt = 5.805 + 12.6784*pmv
! - PMV = 0 (gt depends on clothing insulation, function iso_Ridder): 
!            gt = 21.258 - 9.558*clo
! - PMV > 0: 16.826 + 6.163*pmv
!
! The regression presumes the Klima-Michel settings for reference 
! individual and reference environment.

! Function to determine perceived temperature, degree C, from PMV or 
! clothing insulation for comfort (clo) by 3 linear regressions 
! describing the cases "heat load", "comfort achievable", "cold stress". 
! Regression equation determined from in total more than 400000 synoptic 
! observations at the sites 10803 Freiburg, 10908 Feldberg/Schw., 
! 10170 Rostock-Warnemuende.
!
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
      real pmv,clo
!
      if(pmv.le.-0.11) then
         gtregr = 5.805 + 12.6784*pmv
      else
         if(pmv.ge.+0.01) then
            gtregr = 16.826 + 6.163*pmv
         else
            gtregr = 21.258 - 9.558*clo
         endif
      endif
!
      return
      END function gtregr
!
!---------------------------------------------------------------------
      subroutine fanger(tt,tmrt,pa,vau,pb,clo,actlev,eta,pmva,top)
!---------------------------------------------------------------------
!
! FANGER.F90
!
!     SI-VERSION: ACTLEV W m-2, DAMPFDRUCK hPa
!     Berechnet das aktuelle Predicted Mean Vote nach Fanger
!
! Input variables of argument list:
! ---------------------------------
! - tt   = ambient temperature (C) at screen level
! - tmrt  = mean radiant temperature (C) at screen level
! - pa    = water vapour pressure (hPa) at screen level
! - vau   = wind speed (m/s) 1 m above ground
! - pb    = barometric pressure (hPa) at site (equilvalent to site altitude)
! - clo   = clothing insulation (clo)
! - actlev= individuals activity level per unit surface area (W/m2)
! - eta   = individuals mechanical work load (W/m2)
!
! Output variables of argument list:
! ----------------------------------
! - pmva  = actual Predicted Mean Vote (dimensionless) according 
!           to Fanger corresponding to meteorological (tt,tmrt,pa,vau,pb) 
!           and individual variables (clo, actlev, eta)
! - top   = operative temperature (C)
!
! The case of free convection (vau < 0.1 m/s) is dealt with vau = 0.1 m/s
!
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! source Klima Michel- UP Fanger by Prof. Dr. Gerd Jendritzky 
! code by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typvereinbarung fuer die Eingangsgroessen der Argumentliste
      real tt,tmrt,pa,pb,vau,clo,actlev,eta,top
!     --- Typvereinbarung fuer die Ausgabegroesse der Argumentliste
      real pmva
!
!     --- Typvereinbarung fuer die Variablen des Programms
      real FCL, HC, ACTI, AC, BC, CC, DC, EC, GC, TCL,HR
      real Z1, Z2, Z3, Z4, Z5, Z6
      integer i
!
!     --- Wegen folgender Division muss CLO > 0. sein
      IF(clo.LE.0.) clo=.001
!
!     --- FCL = Oberflaechenvergroesserung durch Bekleidung
      FCL=1.+.15*clo
!
!     --- Der Fall nur freie Konvektion (vau < 0.1 m/s ) wird nicht behandelt
      IF(vau .LT. .1) THEN
         vau = .1
      ENDIF
!
!     --- HC = erzwungene Konvektion
      HC=12.1*SQRT(vau*pb/1013.25)
!
!     --- ACTI = innere Waermeproduktion pro Einheitsflaeche
      ACTI=actlev*(1.-eta)
!
!     --- AC = Mittlere Hauttemperatur
      AC=35.7-.0275*ACTI
!
!     --- Berechnung von Konstanten fuer die folgenden Auswertungen
      BC=.155*clo*3.96*10.**(-8)*FCL
      CC=FCL*HC
      EC=.155*clo
      DC=(1.+EC*CC)/BC
      GC=(AC+BC*(tmrt+273.2)**4+EC*CC*tt)/BC
!
!     --- Berechnung der Oberflaechentemperatur (TCL) der Bekleidung durch
!         Newton-Naeherung mit Lufttemperatur als nullter Naeherung
      TCL = tt
      DO  I= 1, 3
         TCL=TCL-((TCL+273.2)**4+TCL*DC-GC)/(4.*(TCL+                   &
     &   273.2)**3+DC)
      ENDDO
!
!     --- Empirischer Faktor fuer die Anpassung der Waermebilanzgleichung
!         an die psycho-physikalische Skala (FORMEL 40 AUS FANGER)
      Z1 = (.303*EXP(-.036*actlev)+.0275)
!
!     --- Z2 = Wasserdampfdiffusion durch die Haut
      Z2 = .31*(57.3-.07*ACTI-pa)
!
!     --- Z3 = Verdunstung von Schweiss auf der Hautoberflaeche
      Z3 = .42*(ACTI-58.)
!
!     --- Z4  Latenter Waermeverlust durch Atmung
      Z4 = .0017*actlev*(58.7-pa)+.0014*actlev*(34.-tt)
!
!     --- Z5 = Strahlungswaermeverlust
      Z5 = 3.96E-8*FCL*((TCL+273.2)**4-(tmrt+273.2)**4)
      if(abs(tcl-tmrt).gt.0.) then
         hr=z5/fcl/(tcl-tmrt)
      else
         hr=0.
      endif
!
!     --- Z6 = Waermeverlust durch erzwung. Konvekt. CC*(TCL-TT)
      Z6 = CC*(TCL-tt)
!
!     --- Predicted Mean Vote
      pmva=Z1*(ACTI-Z2-Z3-Z4-Z5-Z6)
!
!     --- operative temperatur
      top=(hr*tmrt+hc*tt)/(hr+hc)
!
      return
      END subroutine fanger
!
!---------------------------------------------------------------------
      real function deltapmv(pmva,tt2m,el2m,svp_tt,tmrt,vau1m,nerr)
!---------------------------------------------------------------------
!
! For pmva > 0 and clo =0.5 the increment (deltapmv) is calculated  
! that converts pmva into Gagge's et al. (1986) PMV*.
!
! Input variables of argument list:
! ---------------------------------
! - pmva  = actual Predicted Mean Vote (PMV) according to Fanger
! - tt2m  = ambient temperature (C) at screen level
! - el2m  = water vapour pressure (hPa) at screen level
! - svp_tt= saturation water vapour pressure (hPa) at tt2m
! - tmrt  = mean radiant temperature (C) at screen level
! - vau1m = wind speed (m/s) 1 m above ground
!
! Output variables of argument list:
! ----------------------------------
! - nerr  = error status: 
!            0 = o.k. 
!           -2 = pmva outside valid regression range
!           -3 = rel. humidity set to 5 % or 95 %, respectively
!           -4 = deltapmv set to avoid pmvs < 0
!
! Method:
! -------
! Multiple linear regression in input variables. Valid exclusively for 
! Klima-Michel definition (age= 35 years, weight= 75 kg, tallness= 1.75 m,
! activity level per unit body surface= 134.682 W/m2 (2.3 met), 
! mechanical work load = 0 W/m2). Valid regression thresholds: 0 < pmva < 6.
! Standard error = 0.24 PMV, coefficient of determination = 96%.
!
! Reference for basic method:
! ---------------------------
! - Gagge AP, Fobelets AP, Berglund PE (1986): A Standard Predictive 
!     Index of Human Response to the Thermal Environment. 
!     ASHRAE Trans., Vol. 92: 709-731.
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typvereinbarung fuer Argumentliste / types of the argument list
      real      :: pmva,tt2m,el2m,svp_tt,tmrt,vau1m
      integer   :: nerr
!
!     --- Typvereinbarung fuer unabhaengige Variable
!         type of programme variables
      real      :: pmv,pa_p50,pa,apa,dapa,sqvel,ta,dtmrt
      real      :: p10,p95,gew,gew2,dpmv_1,dpmv_2,pmvs
      integer   :: nreg
      real      :: bpmv(0:7),bpa_p50(0:7),bpa(0:7),bapa(0:7),bdapa(0:7)
      real      :: bsqvel(0:7),bta(0:7),bdtmrt(0:7),aconst(0:7)
!
      data bpmv /                                                       &
     &-0.0556602,-0.1528680,-0.2336104,-0.2789387,-0.3551048,-0.4304076,&
     &-0.4884961,-0.4897495/
      data bpa_p50/                                                     &
     &-0.1607154,-0.4177296,-0.4120541,-0.0886564,+0.4285938,+0.6281256,&
     &+0.5067361,+0.3965169/
      data bpa /                                                        &
     &+0.0580284,+0.0836264,+0.1009919,+0.1020777,+0.0898681,+0.0839116,&
     &+0.0853258,+0.0866589/
      data bapa/                                                        &
     &-1.7838788,-2.9306231,-1.6350334,+0.6211547,+3.3918083,+5.5521025,&
     &+8.4897418,+16.6265851/
      data bdapa/                                                       &
     &+1.6752720,+2.7379504,+1.2940526,-1.0985759,-3.9054732,-6.0403012,&
     &-8.9437119,-17.0671201/
      data bsqvel/                                                      &
     &-0.0315598,-0.0286272,-0.0009228,+0.0483344,+0.0992366,+0.1491379,&
     &+0.1951452,+0.2133949/
      data bta/                                                         &
     &+0.0953986,+0.1524760,+0.0564241,-0.0893253,-0.2398868,-0.3515237,&
     &-0.5095144,-0.9469258/
      data bdtmrt/                                                      &
     &-0.0004672,-0.0000514,-0.0018037,-0.0049440,-0.0069036,-0.0075844,&
     &-0.0079602,-0.0089439/
      data aconst/                                                      &
     &+1.8686215,+3.4260713,+2.0116185,-0.7777552,-4.6715853,-7.7314281,&
     &-11.7602578,-23.5934198/
!
!     --- Einhaltung des Bereichs pruefen, fuer den die Regression gilt
!         test for compliance with regression range
      if(pmva.lt.-1.0.or.pmva.gt.7.0) then
         nerr=-2
      else
         nerr=0
      endif
!
!     --- Unabhaengige Variable initialisieren / initialise
!     ... klassisches PMV / classical PMV
      pmv=pmva
!     ... aktueller Dampfdruck und sein Logarithmus
!         water vapour pressure of air 
      p10=0.05*svp_tt
!      p95=0.95*svp_tt
      p95=1.00*svp_tt
      if(el2m.ge.p10.and.el2m.le.p95) then
         pa=el2m
      else
         nerr=-3
         if(el2m.lt.p10) then
!           ... wegen Voraussetzungen des Regressionsansatzes: rF>=5 %
!               due to conditions of regression: r.H. >= 5 %
            pa=p10
         else
!           ... wegen Voraussetzungen des Regressionsansatzes: rF<=95 %
!               due to conditions of regression: r.H. <= 95 %
            pa=p95
         endif
      endif
      if(pa.gt.0.) then
!        ... natural logarithm of pa
         apa=alog(pa)
      else
         apa= -5.
      endif
!     ... Quotient aktueller Dampfdruck zu dem von 50 % Feuchte bei tt2m,
!         Logarithmus des Quotienten
!         ratio actual water vapour pressure to that of a r.H. of 50 %
      pa_p50=0.5*svp_tt
      if(pa_p50.gt.0..and.pa.gt.0.) then
         dapa=apa-alog(pa_p50)
         pa_p50=pa/pa_p50
      else
         dapa=-5.
         pa_p50=0.
      endif
!     ... Quadratwurzel der Windgeschwindigkeit
!         square root of wind velocity
      if(vau1m.ge.0.) then
         sqvel=sqrt(vau1m)
      else
         sqvel=0.
      endif
!     ... Lufttemperatur / air temperature
      ta=tt2m
!     ... Differenz Strahlungs- zu Lufttemperatur
!         difference mean radiation to air temperature
      dtmrt=tmrt-ta
!
!     --- Auswahl des Regressionsansatzes
!         select the valid regression coefficients
      nreg=int(pmv)
      if(nreg.lt.0) then
!        ... Funktionswert fuer pmv-Werte <= -1.
!            value of the function in the case pmv <= -1
         deltapmv=0.
         return
      endif
      gew=amod(pmv,1.)
      if(gew.lt.0.) gew=0.
      if(nreg.gt.5) then
!         nreg=6
         nreg=5
         gew=pmv-5.
         gew2=pmv-6.
         if(gew2.gt.0) then
            gew=(gew-gew2)/gew
         endif
!         gew=0.
      endif
!
!     --- Regressionsansatz gueltig fuer 0. <= pmv <= 6.0
!         regression valid for 0. <= pmv <= 6.
      dpmv_1=                                                           &
     & + bpa(nreg)*pa                                                   &
     & + bpmv(nreg)*pmv                                                 &
     & + bapa(nreg)*apa                                                 &
     & + bta(nreg)*ta                                                   &
     & + bdtmrt(nreg)*dtmrt                                             &
     & + bdapa(nreg)*dapa                                               &
     & + bsqvel(nreg)*sqvel                                             &
     & + bpa_p50(nreg)*pa_p50                                           &
     & + aconst(nreg)
!
      dpmv_2=0.
      if(nreg.lt.6) then
         dpmv_2=                                                        &
     &    + bpa(nreg+1)*pa                                              &
     &    + bpmv(nreg+1)*pmv                                            &
     &    + bapa(nreg+1)*apa                                            &
     &    + bta(nreg+1)*ta                                              &
     &    + bdtmrt(nreg+1)*dtmrt                                        &
     &    + bdapa(nreg+1)*dapa                                          &
     &    + bsqvel(nreg+1)*sqvel                                        &
     &    + bpa_p50(nreg+1)*pa_p50                                      &
     &    + aconst(nreg+1)
      endif
!
!     --- Funktionswert / value of the function
      deltapmv=(1.-gew)*dpmv_1+gew*dpmv_2
      pmvs=pmva+deltapmv
      if((pmvs).lt.0.) then
!        ... negatives pmv* unterbinden: Komplikation mit Kleidungsisolation
!            prevent negative pmv* due to problems with clothing insulation
         nerr=-4
         if(pmvs.gt.-0.11) then
!           ... Grenzwert -0.11 aus Funktion gtregr fuer 1.75 clo
!               threshold from function gtregr for winter clothing insulation
            deltapmv=deltapmv+0.11
         else
!           ... pmvs auf "0" setzen, Funktion gtregr: 0.50 clo fuer Komfort
!               set pmvs to "0" for compliance with summer clothing insulation
            deltapmv=-1.*pmva
         endif
      endif
!
      return
      END function deltapmv
!
!---------------------------------------------------------------------
      real function gschwuel(gtc,dgtcm,dgtcstd)
!---------------------------------------------------------------------
!
! Sultriness: 
! The function "gschwuel" returns a threshold value to perceived 
! temperature allowing to decide whether the actual perceived temperature 
! is linked to perecption of sultriness. The threshold values depends 
! on the Fanger's classical PMV, expressed here as perceived temperature 
! gtc. It is derived from all synoptic observations 1966-1995 of the sites 
! 10170 Rostock-Warnemuende and 10803 Freiburg showing heat load. The 
! threshold value is valid for subjects acclimatised to Central European 
! climate conditions. It includes all effective influences as air 
! temperature, relative humidity, mean radiant temperature, and wind 
! velocity:
! - defined only under warm conditions: gtc based on 0.5 (clo) and
!                                       gtc > 16.826 (C)
! - undefined: gschwuel set at +99.
!
! Input of the argument list:
! ---------------------------
! - gtc    = classical perceived temperature: Base is Fanger's PMV as  
!            input to function gtregr
!
! Additional output variables of argument list:
! ---------------------------------------------
! - dgtcm  = mean deviation gtc (classical gt) to gt* (rational gt 
!            calculated based on Gagge's rational PMV*)
! - dgtcstd= dgtcm plus its standard deviation times a factor 
!            determining the significance to perceive sultriness
!
! Method:
! -------
! Mean deviation (dgtcm) between rational perceived temperature (gt*) 
! compared to classical (gtc) calculated based on Fanger's PMV and its 
! its standard deviation (dgtcstd) using a third order polynomial in 
! gtc. Base of assessment: SYNOPs Freiburg and Rostock-Warnemuende 
! (1966-1995) with gtc > 16.826 (C), clothing insulation 0.5 (clo).
! Coefficient of determination of 1 K bins in gtc compared to Savitzky-
! Golay filtered means:
! - dgtcm  = 99.828 %
! - dgtcstd= 99.505 %
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- type of input variable of the argument list
      real      :: gtc
!     --- type of output variables of the argument list
      real      :: dgtcm,dgtcstd
!
!     --- types of coefficients mean deviation: third order polynomial
      real,parameter:: dgtcka=+7.5776086, dgtckb=-0.740603,             &
     &                 dgtckc=+0.0213324, dgtckd=-0.00027797237
!
!     --- types of coefficients mean deviation plus standard deviation
!     ... regression coefficients: third order polynomial
      real,parameter:: dgtcsa=+0.0268918,dgtcsb=+0.0465957,             &
     &                 dgtcsc=-0.00054709752, dgtcsd=+0.0000063714823
!     ... factor to mean standard deviation defining significance for 
!         sultriness
      real,parameter:: faktor=1.
!
!     --- initialise
      gschwuel=+99.
      dgtcm=0.
      dgtcstd=999999.
!
      if(gtc.lt.16.826.or.gtc.gt.56.) then
!        ... unallowed classical PMV/gtc
         return
      endif
!
!     --- mean deviation dependent on gtc
      dgtcm= dgtcka + dgtckb*gtc + dgtckc*gtc**2. + dgtckd*gtc**3.
!
!     --- mean deviation plus its standard deviation
      dgtcstd= dgtcsa + dgtcsb*gtc + dgtcsc*gtc**2. + dgtcsd*gtc**3.
!
!     --- value of the function
      gschwuel= dgtcm + faktor*dgtcstd
      if(abs(gschwuel).gt.99.) gschwuel=+99.
!
      return
      END function gschwuel
!
!-----------------------------------------------------------------------
      real function dpmv_kalt(pmva,tt2m,vau1m,tmrt,nerr)
!-----------------------------------------------------------------------
!
! Multiple linear regression to calculate an increment delta_kalt, 
! to adjust Fanger's classical PMV (pmva) by Gagge's 2 node model, 
! applying Fanger's convective heat transfer coefficient, hcf.
! Wind velocitiy of the reference environment is 0.10 m/s 
!
! Method:
! -------
! - multiple linear regression, coefficients by SPSS (30.08.2006) of fields
!   = air temperature: +10 to -80, C, increment: 1 K
!   = dtmrt: -30 to +30 K, increment: 1 K
!   = relative wind velocity 1 m above ground: 0.5 to 60, m/s, 28 non 
!     linear increments
!   = pmva dependent on calculation based on above given variations
! - determine the intersection points of the regression lines of the three
!   thermophysiological segments in terms of Fanger's PMV dependent on the
!   the given meteorological input
!
! Input:
! ------
! - pmva  : Fanger's classical predicted mean vote
! - tt2m  : air temperature, C, 2 m above ground
! - vau1m : relative wind velocity 1 m above ground, m/s
! - tmrt  : mean radiant temperature, C
!
! Output:
! -------
! - dpmv_kalt: increment to adjust pmva according to the results of Gagge's 
!              2 node model depending on the input
! - nerr     : error indicator: 0 = o.k., +1 = denominator for intersection = 0
!
! Required subroutines and functions:
! -----------------------------------
! - dpmv_adj: function, adjust to Fanger's operative temperature
!
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
!
      implicit none
!
!     --- type of input arguments
      real, intent(in) :: pmva,tt2m,vau1m,tmrt
!     ... type of output argument
      integer :: nerr
!
!     --- type of function
!      real :: dpmv_adj
!
!     --- type of program variables
      integer, parameter :: n_bin= 3
      real :: delta_kalt(n_bin),pmv_cross(n_bin-1)
      real :: reg_a(n_bin),reg_b(n_bin),coeff(n_bin,5),r_nenner
      real :: pmvc,ta,dtmrt,sqrt_v1m
      integer :: i,j,i_bin
!
!     --- coefficient of the 3 regression lines
!       1:const   2:*pmvc  3:*ta      4:*sqrt_v1m  5:*dtmrt
      data (coeff(1,j),j=1,5) /                                         &
     & +0.161,    +0.130,  -1.125E-03, +1.106E-03, -4.570E-04 /
     data (coeff(2,j),j=1,5) /                                          &
     & +0.795,    +0.713,  -8.880E-03, -1.803E-03, -2.816E-03 /
      data (coeff(3,j),j=1,5) /                                         &
     & +0.05761,  +0.458,  -1.829E-02, -5.577E-03, -1.970E-03 /
!
!     --- initialise
      nerr=0
      dpmv_kalt=0.
      pmvc=pmva
      ta=tt2m
      dtmrt=tmrt - tt2m
      sqrt_v1m=vau1m
      if(sqrt_v1m.lt.0.10) then
         sqrt_v1m=0.10
      else
         sqrt_v1m=sqrt(sqrt_v1m)
      endif
!
      do i=1,n_bin
         delta_kalt(i)= 0.
         if(i.lt.n_bin) then
            pmv_cross(i)= pmvc
         endif
      enddo
!
      do i=1,n_bin
!        ... regression constant for given meteorological variables
         reg_a(i)= coeff(i,1) + coeff(i,3)*ta + coeff(i,4)*sqrt_v1m +   &
     &             coeff(i,5)*dtmrt 
         delta_kalt(i)= reg_a(i) + coeff(i,2)*pmvc
      enddo
!
!     --- intersection points of regression lines in terms of Fanger's PMV
      do i=1,n_bin-1
         r_nenner= coeff(i,2)-coeff(i+1,2)
         if(abs(r_nenner).gt.0.00001) then
            pmv_cross(i)=(reg_a(i+1)-reg_a(i)) / r_nenner
         else
            nerr=1
            return
         endif
      enddo
!
      i_bin=3
      do i=1,n_bin-1
         if(pmva.gt.pmv_cross(i)) then
            i_bin=i
            exit
         endif
      enddo
!     ... adjust to operative temperature scaled according 
!         to classical PMV (Fanger)
      dpmv_kalt=delta_kalt(i_bin) - dpmv_adj(pmva)
!
      return
      end function dpmv_kalt
!
!-----------------------------------------------------------------------
      real function dpmv_adj(pmva)
!-----------------------------------------------------------------------
!
! Calculates the summand dpmv_adj adjusting to the operative temperature 
! scaled according to classical PMV (Fanger)
! Reference environment: v_1m = 0.10 m/s, dTMRT = 0 K, r.h. = 50 %
!
! Argumentlist:
! -------------
! - pmva (input)     : Fanger's classical PMV 
! - dpmv_adj (output): summand to adjust to operative temperature 
!                      calculated according to Fanger
!
! Method: Second order polynomial for 3 bins of pmva
! -------
! - i_bin = 1: bin for tsk > Fanger's tsk_comf
! - i_bin = 2: bin for tsk < Fanger's tsk_comf and store >= 0 W/m2
! - i_bin = 3: bin for tsk < Fanger's tsk_comf and store  < 0 W/m2
! 
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
!
      implicit none
!
      real, intent(in) :: pmva
      integer, parameter :: n_bin=3, n_ca=0,n_ce=3
      real, dimension (n_bin,n_ca:n_ce) :: coef
!
      real       :: pmv
      integer    :: i,i_bin
!
!                                  range_1   range_2    range_3
      data (coef(i,0),i=1,n_bin) /+0.0941540,-0.1506620,-0.0871439/
      data (coef(i,1),i=1,n_bin) /+0.0783162,-1.0612651,+0.1695040/
      data (coef(i,2),i=1,n_bin) /+0.1350144,-1.0049144,-0.0167627/
      data (coef(i,3),i=1,n_bin) /+0.1104037,-0.2005277,-0.0003230/
!
      if(pmva.le.-2.1226) then
         i_bin=3
      else if(pmva.le.-1.28) then
         i_bin=2
      else
         i_bin=1
      endif
!
      dpmv_adj=coef(i_bin,n_ca)
      pmv=1.
      do i=n_ca+1,n_ce
         pmv=pmv*pmva
         dpmv_adj=dpmv_adj + coef(i_bin,i)*pmv
      enddo
!
      return
      end function dpmv_adj
!
!---------------------------------------------------------------------
      real function ireq_neutral(gt,ireq_minimal,nerr)
!---------------------------------------------------------------------
!
! Based on perceived temperature (gt) as input, the function determines 
! the required clothing insulation (clo) for thermally neutral conditions 
! (neither body cooling nor body heating). It is related to the Klima- 
! Michel activity level (134.682 W/m2). IREQ_neutral is only defined 
! for gt < 10 (C)
! 
! Additional output via the argument list: 
! - ireq_minimal: the minimal required clothing insulation (clo) to keep   
!                 the maximum acceptable body cooling.
! - nerr: error status: 0 = o.k., +1 = ireq_neutral < 0.48, set at 0.5
!
! Method:
! -------
! ISO - TR 11079:1993 (E). Linear regressions to convert from gt defined 
! for reference wind speed of 0.1 m/s to gt0.2 defined for a reference of 
! 0.2 m/s. The regression is valid for the range -50 < gt < +50 (C)
!
! Reference:
! ----------
! ISO (1993): Evaluation of cold environments - Determination of required
!   clothing insulation (IREQ). Technical Report ISO TR 11079:1993 (E), first
!   edition. International Organization for Standardization, Geneva.
! 
!
! F_IREQ.F90:
! Berechnet anhand der gefuehlten Temperatur die thermische Isolation der
! Bekleidung, clo, die notwendig ist, um Bedingungen thermischer Neutralitaet
! zu gewaehrleisten. Gibt im Argument IREQ_MINIMAL zusaetzlich die
! geringere thermische Isolation der Bekleidung aus, die zu einer hoechsten,
! unter Arbeitbedingungen gerade noch tolerable Auskuehlung der Koerpers
! fuehrt.
!
! Verfahren:
! ----------
! Genutzt werden die Diagramme in ISO TR 11079, 1993 (E) fuer die Standard-
! bedingungen des Klima-Michels: Metabolische Rate 134.7 W/m2. Die
! Isolationswerte sind abhaengig von der operativen Temperatur, Grad Celsius,
! und gelten fuer eine Windgeschwindigkeit von 0.2 m/s sowie eine relative
! Feuchte von 50 %. Der Geltungsbereich der Kurven liegt zwischen einer
! operativen Temperatur von -50 bis +10 Grad Celsius. Die Feuchtigkeit spielt
! in diesem Bereich keine entscheidende Rolle in der Waermehaushaltsgleichung
! des Menschen mehr, so dass die Diagramme in diesem Bereich mit geringem
! Fehler auf alle vorkommenden Feuchten anwendbar ist.
!
! Die fuer die gefuehlte Temperatur definierte Standardumgebung ist tmrt=
! Lufttemperatur und Windgeschwindigkeit v = 0.1 m/s, die Luftfeuchte der
! Standardumgebung ist identisch mit der der aktuellen Umgebung. Wegen der
! Vorgabe mittlere Strahlungstemperatur = Lufttemperatur ist die gefuehlte
! Temperatur identisch mit der operativen Temperatur in dieser
! Standardumgebung. Das Problem in der Uebernahme der Diagramme aus ISO TR
! 11079 fuer die Berechnung der notwendigen Kleidung bei gegebener gefuehlter
! Temperatur liegt in der unterschiedlichen Bezugswindgeschwindigkeit.
! Berechnungen zeigten, dass nach Definition der gefuehlten Temperatur mit
! 0.1 m/s, sich im kalten Bereich gefuehlte Temperaturen ca. 10 % unter denen
! ergeben, wenn die gefuehlte Temperatur fuer eine Bezugswindgeschwindigkeit
! von 0.2 m/s definiert waere. Die sich fuer die gefuehlte Temperatur
! ergebenden Abweichungen aufgrund der unterschiedlichen Windgeschwindigkeiten
! sind linear; getestet fuer Lufttemperatur= mittlere Strahlungstemperatur
! von -50 bis +50 Grad Celsius. Es gilt die Regressionsbeziehung:
!
!     gt0.2 = 1.8788 + 0.9296*gt0.1
!
! Mit gt0.2 hat man eine gefuehlte Temperatur mit der Bezugswindgeschwindigkeit
! v= 0.2 m/s. Diese ist identisch mit der operativen Temperatur, top0.2, in
! der Referenzumgebung. Fuer die metabolische Rate des Klima-Michels wurden
! folgende Regressionsgleichungen aus den Diagrammen von ISO TR 11079
! abgeleitet:
!
! - ireq_neutral=1.62 - 0.0564*top0.2
! - ireq_minimal=1.26 - 0.0588*top0.2
!
! Die Werte stimmen im Bereich der gefuehlten Temperaturen, die Werte zwischen
! 0.50 und 1.75 clo der Bekleidungisolation verlangen, absolut brauchbar mit
! denen ueberein, die mit der Funktion f_cloc.for" erzeugt werden. Bei Werten
! der Funktion "f_cloc.for" von 4 bis 6 clo liegen die Werte fuer IREQ_NEUTRAL
! 25 bis 30 %  niedriger. Moeglicher Grund sind die Modellunterschiede
! zwischen Fanger und IREQ. IREQ entspricht etwa den Modellansaetzen von Gagge
! et al. (1986), thermische Neutralitaet wird allerdings fuer eine mittlere
! Hauttemperatur angesetzt, die nach Fanger berechnet wird. Die Unterschiede
! zwischen den beiden Modellansaetzen duerfte fuer die Differenzen
! verantwortlich sein. Zwischen IREQ_NEUTRAL und IREQ_MINIMAL ergeben sich die
! folgenden linearen Beziehungen zu den Ergebnissen mit der Funktion "f_cloc",
! die auf Bezugsgeschwindigkeit v= 0.1 beruhen:
!
! - ireq_neutral=0.15903 + 0.6689*cloc
! - ireq_minimal=-0.2633 + 0.6974*cloc
!
! Achtung:
! ========
! Bei Aenderungen der Bezugsgeschwindigkeit ist die Regressionsgleichung, die
! die operative Temperatur auf v= 0.2 m/s umrechnet, zu aendern
!
! Argumente:
! ----------
! - gt  : gefuehlte Temperatur, Grad Celsius, Bezugsgeschwindigkeit v= 0.1 m/s.
! - ireq_minimal: (Ausgabegroesse). Thermische Isolation der Bekleidung, clo,
!         die zu einer gerade noch tolerierbaren Auskuehlung fuehrt.
! - nerr: Fehlerstatus (integer): 0 = normal, +1 = ireq_neutral hat einen
!         Wert der Kleidungsisolation < 0.5 clo erreicht und wurde auf 0.5
!         clo gesetzt.
!
! Funktionswert:
! --------------
! - ireq_neutral: Thermische Isolation der Bekleidung, die zu thermischer
!         Neutralitaet des Koerpers fuehrt.
!
!*********************************************************************
!
! DEUTSCHER WETTERDIENST - Medizin-Meteorologie - Human-Biometeorology
! Concept and code written by Dr. Henning Staiger
!
!*********************************************************************
!
      implicit none
!
!     --- Typvereinbarung fuer die Argumentliste
      real      :: gt,ireq_minimal
      integer   :: nerr
!
!     --- Typvereinbarungen fuer Variable des Programms
      real      :: top02
!
!     --- initialise
      nerr=0
!
!     --- convert perceived temperature from basis 0.1 m/s to basis 0.2 m/s
      top02=1.8788 + 0.9296*gt
!
!     --- IREQ neutral conditions (thermal comfort)
!         ... value of the function
      ireq_neutral=1.62 - 0.0564*top02
!
!     --- regression only defined for gt <= 10 (C)
      if(ireq_neutral.lt.0.5) then
         if(ireq_neutral.lt.0.48) then
            nerr=1
         endif
         ireq_neutral=0.5
      endif
!
!     --- minimal required clothing insulation:maximal acceptable body cooling
      ireq_minimal=1.26 - 0.0588*top02
      if(nerr.gt.0) then
         ireq_minimal=ireq_neutral
      endif
!
      return
      END function ireq_neutral
!
end module GT
